# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from eve.utils import config
from flask import g, current_app as app

import superdesk
from superdesk import get_resource_service
from superdesk.activity import (
    ACTIVITY_CREATE,
    ACTIVITY_EVENT,
    ACTIVITY_UPDATE,
    notify_and_add_activity,
    ACTIVITY_DELETE,
)
from superdesk.errors import SuperdeskApiError
from superdesk.io import allowed_feeding_services, allowed_feed_parsers, get_feeding_service
from superdesk.metadata.item import CONTENT_STATE, CONTENT_TYPE
from superdesk.notification import push_notification
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.utc import utcnow
from superdesk.utils import required_string

logger = logging.getLogger(__name__)


class IngestProviderResource(Resource):
    """Ingest provider model

    :param name: provider name
    :param source: populates item source field
    :param feeding_service: feeding service name
    :param feed_parser: feed parser name
    :param content_types: list of content types of items to ingest from provider
    :param allow_remove_ingested: allow deleting of items from ingest
    :param content_expiry: ttl for ingested items in minutes
    :param config: provider specific config
    :param private: can contain any data useful for provider (e.g. to manage feeds position)
    :param ingested_count: number of items ingested so far
    :param tokens: auth tokens used by provider
    :param is_closed: provider closed status
    :param update_schedule: update schedule, will run every x hours x minutes x seconds
    :param idle_time: usual idle time for provider, if there is no item after that it will warn
    :param last_updated: last update timestamp
    :param rule_set: rule sets used when ingesting
    :param routing_scheme: routing scheme used when ingesting
    :param notifications: set when notification should be sent for this provider
    :param last_closed: info when and by whom provider was closed last time
    :param last_opened: info when and by whom provider was opened last time
    :param critical_errors: error codes which are considered critical and should close provider
    """

    mongo_indexes = {
        "name_1": ([("name", 1)], {"unique": True}),
    }

    def __init__(self, endpoint_name, app, service, endpoint_schema=None):
        self.schema = {
            "name": {"type": "string", "required": True, "nullable": False, "empty": False, "iunique": True},
            "source": required_string,
            "feeding_service": {"type": "string", "required": True, "allowed": allowed_feeding_services},
            "feed_parser": {"type": "string", "nullable": True, "allowed": allowed_feed_parsers},
            "content_types": {"type": "list", "default": tuple(CONTENT_TYPE), "allowed": tuple(CONTENT_TYPE)},
            "allow_remove_ingested": {"type": "boolean", "default": False},
            "content_expiry": {"type": "integer", "default": app.config["INGEST_EXPIRY_MINUTES"]},
            "config": {
                "type": "dict",
                "schema": {},
                "allow_unknown": True,
            },
            "private": {
                "type": "dict",
                "schema": {},
                "allow_unknown": True,
            },
            "ingested_count": {"type": "integer"},
            "accepted_count": {"type": "integer"},
            "tokens": {
                "type": "dict",
                "schema": {},
                "allow_unknown": True,
            },
            "is_closed": {"type": "boolean", "default": False},
            "update_schedule": {
                "type": "dict",
                "schema": {
                    "hours": {"type": "integer"},
                    "minutes": {"type": "integer", "default": 5},
                    "seconds": {"type": "integer"},
                },
            },
            "idle_time": {
                "type": "dict",
                "schema": {
                    "hours": {"type": "integer"},
                    "minutes": {"type": "integer"},
                },
            },
            "last_updated": {"type": "datetime"},
            "last_ingested_id": {"type": "string"},  # this id is ingest provider internal value
            "last_item_update": {"type": "datetime"},
            "rule_set": Resource.rel("rule_sets", nullable=True),
            "routing_scheme": Resource.rel("routing_schemes", nullable=True),
            "notifications": {
                "type": "dict",
                "schema": {
                    "on_update": {"type": "boolean", "default": True},
                    "on_close": {"type": "boolean", "default": True},
                    "on_open": {"type": "boolean", "default": True},
                    "on_error": {"type": "boolean", "default": True},
                },
                "default": {
                    "on_update": True,
                    "on_close": True,
                    "on_open": True,
                    "on_error": True,
                },
            },
            "last_closed": {
                "type": "dict",
                "schema": {
                    "closed_at": {"type": "datetime"},
                    "closed_by": Resource.rel("users", nullable=True),
                    "message": {"type": "string"},
                },
            },
            "last_opened": {
                "type": "dict",
                "schema": {"opened_at": {"type": "datetime"}, "opened_by": Resource.rel("users", nullable=True)},
            },
            "critical_errors": {"type": "dict", "valueschema": {"type": "boolean"}},
            "skip_config_test": {
                "type": "boolean",
                "nullable": True,
            },
            "url_id": {
                "type": "string",
                "nullable": True,
            },
        }

        self.item_methods = ["GET", "PATCH", "DELETE"]
        self.privileges = {"POST": "ingest_providers", "PATCH": "ingest_providers", "DELETE": "ingest_providers"}
        self.etag_ignore_fields = ["last_updated", "last_item_update", "last_closed", "last_opened"]

        super().__init__(endpoint_name, app, service, endpoint_schema=endpoint_schema)


class IngestProviderService(BaseService):
    def __init__(self, datasource=None, backend=None):
        super().__init__(datasource=datasource, backend=backend)

    def _set_provider_status(self, doc, message=""):
        user = getattr(g, "user", None)
        if doc.get("is_closed", True):
            doc["last_closed"] = doc.get("last_closed", {})
            doc["last_closed"]["closed_at"] = utcnow()
            doc["last_closed"]["closed_by"] = user["_id"] if user else None
            doc["last_closed"]["message"] = message
        else:
            doc["last_opened"] = doc.get("last_opened", {})
            doc["last_opened"]["opened_at"] = utcnow()
            doc["last_opened"]["opened_by"] = user["_id"] if user else None

    @property
    def user_service(self):
        return get_resource_service("users")

    def on_create(self, docs):
        for doc in docs:
            content_expiry = doc.get("content_expiry", 0)
            if content_expiry == 0:
                doc["content_expiry"] = app.config["INGEST_EXPIRY_MINUTES"]
                self._set_provider_status(doc, doc.get("last_closed", {}).get("message", ""))
            elif content_expiry < 0:
                doc["content_expiry"] = None
            self._test_config(doc)

    def on_created(self, docs):
        for doc in docs:
            notify_and_add_activity(
                ACTIVITY_CREATE,
                "Created Ingest Channel {{name}}",
                self.datasource,
                item=None,
                user_list=self.user_service.get_users_by_user_type("administrator"),
                name=doc.get("name"),
                provider_id=doc.get("_id"),
            )
            push_notification("ingest_provider:create", provider_id=str(doc.get("_id")))
        logger.info("Created Ingest Channel. Data:{}".format(docs))

    def on_update(self, updates, original):
        try:
            content_expiry = updates["content_expiry"]
        except KeyError:
            content_expiry = None
        else:
            if content_expiry == 0:
                content_expiry = app.config["INGEST_EXPIRY_MINUTES"]
            elif content_expiry < 0:
                content_expiry = None
            updates["content_expiry"] = content_expiry
        if "is_closed" in updates and original.get("is_closed", False) != updates.get("is_closed"):
            self._set_provider_status(updates, updates.get("last_closed", {}).get("message", ""))
        if "config" in updates:
            self._test_config(updates, original)

    def on_updated(self, updates, original):
        do_notification = updates.get("notifications", {}).get(
            "on_update", original.get("notifications", {}).get("on_update", True)
        )
        notify_and_add_activity(
            ACTIVITY_UPDATE,
            "updated Ingest Channel {{name}}",
            self.datasource,
            item=None,
            user_list=self.user_service.get_users_by_user_type("administrator") if do_notification else None,
            name=updates.get("name", original.get("name")),
            provider_id=original.get("_id"),
        )

        if updates.get("is_closed", False) != original.get("is_closed", False):
            status = ""
            do_notification = False

            if updates.get("is_closed"):
                status = "closed"
                do_notification = updates.get("notifications", {}).get(
                    "on_close", original.get("notifications", {}).get("on_close", True)
                )
            elif not updates.get("is_closed"):
                status = "opened"
                do_notification = updates.get("notifications", {}).get(
                    "on_open", original.get("notifications", {}).get("on_open", True)
                )

            notify_and_add_activity(
                ACTIVITY_EVENT,
                "{{status}} Ingest Channel {{name}}",
                self.datasource,
                item=None,
                user_list=self.user_service.get_users_by_user_type("administrator") if do_notification else None,
                name=updates.get("name", original.get("name")),
                status=status,
                provider_id=original.get("_id"),
            )

        push_notification("ingest_provider:update", provider_id=str(original.get("_id")))
        logger.info("Updated Ingest Channel. Data: {}".format(updates))

    def on_delete(self, doc):
        """
        Overriding to check if the Ingest Source which has received item being deleted.
        """

        if doc.get("last_item_update"):
            raise SuperdeskApiError.forbiddenError("Deleting an Ingest Source after receiving items is prohibited.")

    def on_deleted(self, doc):
        """
        Overriding to send notification and record activity about channel deletion.
        """
        notify_and_add_activity(
            ACTIVITY_DELETE,
            "Deleted Ingest Channel {{name}}",
            self.datasource,
            item=None,
            user_list=self.user_service.get_users_by_user_type("administrator"),
            name=doc.get("name"),
            provider_id=doc.get(config.ID_FIELD),
        )
        push_notification("ingest_provider:delete", provider_id=str(doc.get(config.ID_FIELD)))
        get_resource_service("sequences").delete(
            lookup={"key": "ingest_providers_{_id}".format(_id=doc[config.ID_FIELD])}
        )
        logger.info("Deleted Ingest Channel. Data:{}".format(doc))

    def _test_config(self, updates, original=None):
        provider = original.copy() if original else {}
        provider.update(updates)

        if provider.get("skip_config_test"):
            return

        try:
            service = get_feeding_service(provider["feeding_service"])
        except KeyError:
            return
        service.config_test(provider)


superdesk.workflow_state(CONTENT_STATE.INGESTED)
superdesk.workflow_action(name="ingest")



import unittest

from apps.comments.comments import encode_keys, decode_keys
from apps.comments.user_mentions import get_mentions


class CommentsTestCase(unittest.TestCase):
    def test_user_mentions(self):
        users, desks = get_mentions("hello @petr.jasek and #sports.desk and @a-_.")
        self.assertIn("a-_.", users)
        self.assertIn("petr.jasek", users)
        self.assertIn("sports.desk", desks)

    def test_encode_decode_keys(self):
        doc = {"foo": {"x.y": 1}}
        encode_keys(doc, "foo")
        for key in doc["foo"]:
            self.assertNotIn(".", key)
        decode_keys(doc, "foo")
        self.assertIn("x.y", doc["foo"])
        self.assertEqual(1, doc["foo"]["x.y"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2019 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import abc
import logging

logger = logging.getLogger(__name__)

registered_ai_services = {}


class AIServiceRegisterer(abc.ABCMeta):
    def __call__(cls, *args, **kwargs):
        instance = super().__call__(*args, **kwargs)
        name = instance.name
        if name in registered_ai_services:
            # we log a warning but don't raise an exception because the issue
            # may happen with tests
            logger.warning('"{name}" ai service is already registered'.format(name=name))
            return registered_ai_services[name]

        registered_ai_services[name] = instance
        return instance


class AIServiceBase(metaclass=AIServiceRegisterer):
    """Base class for AI service.

    This class define the attribute and methods mandatory to implement.
    AI services are automatically instanciated and registered, you just need to
    inherit from this class in your module.
    "label" attribute can be used, if it is not present the name will be used as label
    """

    def __init__(self, app):
        pass

    @property
    @abc.abstractmethod
    def name(self):
        pass

    @property
    def label(self):
        return self.name.title()

    @abc.abstractmethod
    def analyze(self, item: dict) -> dict:
        """Analyze article"""
        pass



import flask
import unittest


from superdesk.io.subjectcodes import init_app


class SubjectsTestCase(unittest.TestCase):
    def test_app_subjects(self):
        app = flask.Flask(__name__)
        app.api_prefix = "/test"
        init_app(app)
        with app.app_context():
            app.subjects.register({"01000000": "Foo"})
            self.assertEqual("Foo", app.subjects["01000000"])
            app.subjects.clear()
            with self.assertRaises(KeyError):
                app.subjects["01000000"]



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import json
import datetime
from unittest import mock

from superdesk.publish import init_app
from superdesk.publish.formatters.email_formatter import EmailFormatter
from superdesk.tests import TestCase
from superdesk.utc import utc


@mock.patch("superdesk.publish.subscribers.SubscribersService.generate_sequence_number", lambda self, subscriber: 1)
class EmailFormatterTest(TestCase):
    def setUp(self):
        self.formatter = EmailFormatter()
        # self.base_formatter = Formatter()
        init_app(self.app)

    def test_formatter(self):
        article = {
            "source": "AAP",
            "headline": "This is a test headline",
            "abstract": "Can of beans",
            "byline": "joe",
            "ednote": "Very good story",
            "dateline": {"text": "BERN, July 13  -"},
            "slugline": "slugline",
            "anpa_take_key": "take",
            "subject": [{"qcode": "02011001"}],
            "format": "HTML",
            "type": "text",
            "body_html": "<p>The story body of the story so far</p>",
            "word_count": "1",
            "priority": 1,
            "place": [{"qcode": "VIC", "name": "VIC"}],
            "genre": [],
            "sign_off": "aa/bb",
        }

        article["versioncreated"] = datetime.datetime(
            year=2015, month=1, day=30, hour=2, minute=40, second=56, tzinfo=utc
        )
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        item = json.loads(doc)
        self.assertEqual(item["message_subject"], "This is a test headline")
        self.assertEqual(
            item["message_html"],
            "<html>\n<body>\n<h1>VIC:&nbsp;This is a test headline</h1>\n"
            "Published At : Fri Jan 30 03:40:56 2015\n<br>\n<b>slugline"
            "</b>&nbsp;take&nbsp;\n"
            '<hr>\n<br><font color="red">Very good story</font>'
            "<br>\n<i>Can of beans</i>\n<br>joe\n<br>\n"
            "<p>BERN, July 13  - The story body of the story so far</p>\n<br>\n"
            "AAP&nbsp;aa/bb\n\n</body>\n</html>",
        )
        self.assertEqual(
            item["message_text"],
            "VIC: This is a test headline\nPublished At : Fri Jan 30 03:40:56 2015\n"
            "slugline take\nVery good story\n------------------------------------"
            "----------------------\nCan of beans\n\njoe\nBERN, July 13  - The story"
            " body of the story so far\n\nAAP aa/bb\n",
        )

    def test_preserved_formatter(self):
        article = {
            "source": "AAP",
            "headline": "This is a test headline",
            "abstract": "Can of beans",
            "byline": "joe",
            "dateline": {"text": "BERN, July 13  -"},
            "slugline": "slugline",
            "anpa_take_key": "take",
            "subject": [{"qcode": "02011001"}],
            "format": "preserved",
            "type": "text",
            "body_html": "The story body of the story so far",
            "word_count": "1",
            "priority": 1,
            "place": [{"qcode": "VIC", "name": "VIC"}],
            "genre": [],
            "sign_off": "aa/bb",
        }

        article["versioncreated"] = datetime.datetime(
            year=2015, month=1, day=30, hour=2, minute=40, second=56, tzinfo=utc
        )
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        item = json.loads(doc)
        self.assertEqual(item["message_subject"], "This is a test headline")
        self.assertEqual(item["message_html"], None)

    def test_no_place_formatter(self):
        article = {
            "source": "AAP",
            "headline": "This is a test headline",
            "abstract": "Can of beans",
            "byline": "joe",
            "ednote": "Very good story",
            "dateline": {"text": "BERN, July 13  -"},
            "slugline": "slugline",
            "anpa_take_key": "take",
            "subject": [{"qcode": "02011001"}],
            "format": "HTML",
            "type": "text",
            "body_html": "<p>The story body of the story so far</p>",
            "word_count": "1",
            "priority": 1,
            "genre": [],
            "sign_off": "aa/bb",
        }

        article["versioncreated"] = datetime.datetime(
            year=2015, month=1, day=30, hour=2, minute=40, second=56, tzinfo=utc
        )
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        item = json.loads(doc)
        self.assertEqual(item["message_subject"], "This is a test headline")
        self.assertEqual(
            item["message_html"],
            "<html>\n<body>\n<h1>This is a test headline</h1>\n"
            "Published At : Fri Jan 30 03:40:56 2015\n<br>\n<b>slugline"
            "</b>&nbsp;take&nbsp;\n"
            '<hr>\n<br><font color="red">Very good story</font>'
            "<br>\n<i>Can of beans</i>\n<br>joe\n<br>\n"
            "<p>BERN, July 13  - The story body of the story so far</p>\n<br>\n"
            "AAP&nbsp;aa/bb\n\n</body>\n</html>",
        )
        self.assertEqual(
            item["message_text"],
            "This is a test headline\nPublished At : Fri Jan 30 03:40:56 2015\n"
            "slugline take\nVery good story\n------------------------------------"
            "----------------------\nCan of beans\n\njoe\nBERN, July 13  - The story"
            " body of the story so far\n\nAAP aa/bb\n",
        )

    def test_none_place_formatter(self):
        article = {
            "source": "AAP",
            "headline": "This is a test headline",
            "abstract": "Can of beans",
            "byline": "joe",
            "ednote": "Very good story",
            "dateline": {"text": "BERN, July 13  -"},
            "slugline": "slugline",
            "anpa_take_key": "take",
            "subject": [{"qcode": "02011001"}],
            "format": "HTML",
            "type": "text",
            "body_html": "<p>The story body of the story so far</p>",
            "word_count": "1",
            "priority": 1,
            "place": None,
            "genre": [],
            "sign_off": "aa/bb",
        }

        article["versioncreated"] = datetime.datetime(
            year=2015, month=1, day=30, hour=2, minute=40, second=56, tzinfo=utc
        )
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        item = json.loads(doc)
        self.assertEqual(item["message_subject"], "This is a test headline")
        self.assertEqual(
            item["message_html"],
            "<html>\n<body>\n<h1>This is a test headline</h1>\n"
            "Published At : Fri Jan 30 03:40:56 2015\n<br>\n<b>slugline"
            "</b>&nbsp;take&nbsp;\n"
            '<hr>\n<br><font color="red">Very good story</font>'
            "<br>\n<i>Can of beans</i>\n<br>joe\n<br>\n"
            "<p>BERN, July 13  - The story body of the story so far</p>\n<br>\n"
            "AAP&nbsp;aa/bb\n\n</body>\n</html>",
        )
        self.assertEqual(
            item["message_text"],
            "This is a test headline\nPublished At : Fri Jan 30 03:40:56 2015\n"
            "slugline take\nVery good story\n------------------------------------"
            "----------------------\nCan of beans\n\njoe\nBERN, July 13  - The"
            " story body of the story so far\n\nAAP aa/bb\n",
        )

    def test_none_takekey_ednote(self):
        article = {
            "source": "AAP",
            "headline": "This is a test headline",
            "abstract": "Can of beans",
            "byline": "joe",
            "ednote": None,
            "dateline": {"text": "BERN, July 13  -"},
            "slugline": "slugline",
            "anpa_take_key": None,
            "subject": [{"qcode": "02011001"}],
            "format": "HTML",
            "type": "text",
            "body_html": "<p>The story body of the story so far</p>",
            "word_count": "1",
            "priority": 1,
            "place": None,
            "genre": [],
            "sign_off": "aa/bb",
        }

        article["versioncreated"] = datetime.datetime(
            year=2015, month=1, day=30, hour=2, minute=40, second=56, tzinfo=utc
        )
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        item = json.loads(doc)
        self.assertEqual(item["message_subject"], "This is a test headline")
        self.assertEqual(
            item["message_html"],
            "<html>\n<body>\n<h1>This is a test headline</h1>\n"
            "Published At : Fri Jan 30 03:40:56 2015\n<br>\n<b>slugline"
            "</b>&nbsp;\n<hr>\n"
            "\n<i>Can of beans</i>\n<br>joe\n<br>\n"
            "<p>BERN, July 13  - The story body of the story so far</p>\n<br>\n"
            "AAP&nbsp;aa/bb\n\n</body>\n</html>",
        )
        self.assertEqual(
            item["message_text"],
            "This is a test headline\nPublished At : Fri Jan 30 03:40:56 2015\n"
            "slugline \n\n------------------------------------"
            "----------------------\nCan of beans\n\njoe\nBERN, July 13  - The story"
            " body of the story so far\n\nAAP aa/bb\n",
        )

    def test_subject_cyrilic(self):
        article = {"headline": "Неправильная музыка Джамала Али"}
        seq, doc = self.formatter.format(article, {"name": "Test"})[0]
        item = json.loads(doc)
        self.assertEqual(article["headline"], item["message_subject"])

    def test_paragraphs(self):
        """Test that paragraphs (block elements) are followed by line feed in text version

        SDESK-824 regression test
        """
        article = {
            "source": "AAP",
            "anpa_take_key": "take",
            "subject": [{"qcode": "02011001"}],
            "format": "HTML",
            "type": "text",
            "body_html": "<p>paragraph 1</p><br><br><p>paragraph 2</p><p>paragraph 3</p>",
            "word_count": "1",
            "priority": 1,
            "place": [{"qcode": "VIC", "name": "VIC"}],
            "genre": [],
            "sign_off": "aa/bb",
        }
        article["versioncreated"] = datetime.datetime(
            year=2017, month=2, day=24, hour=16, minute=40, second=56, tzinfo=utc
        )
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]

        item = json.loads(doc)
        self.assertEqual(
            item["message_text"],
            "VIC: \nPublished At : Fri Feb 24 17:40:56 2017\n"
            " take\n\n--------------------------------------"
            "--------------------\n\n\n\nparagraph 1\n\n\n"
            "paragraph 2\nparagraph 3\n\nAAP aa/bb\n",
        )

    def test_dateline_html(self):
        """Check that message_html output is producted even without a dateline

        SDESK-836 regression test
        """
        article = {"format": "HTML", "type": "text", "body_html": "<p>some HTML</p>"}
        _, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        item = json.loads(doc)
        self.assertIsNotNone(item["message_html"])

    def test_featuremedia(self):
        article = {
            "source": "AAP",
            "anpa_take_key": "take",
            "subject": [{"qcode": "02011001"}],
            "format": "HTML",
            "type": "text",
            "body_html": "<p>Test</p>",
            "word_count": "1",
            "priority": 1,
            "place": [{"qcode": "VIC", "name": "VIC"}],
            "genre": [],
            "sign_off": "aa/bb",
            "associations": {
                "featuremedia": {
                    "archive_description": "A woman crosses Bourke Street Mall",
                    "renditions": {
                        "viewImage": {
                            "poi": {"y": 257, "x": 454},
                            "href": "http://localhost:5000/api/upload-raw/5c11ece81d41c89113ed202b?_schema=http",
                            "media": "5c11ece81d41c89113ed202b",
                            "height": 390,
                            "mimetype": "image/jpeg",
                            "width": 640,
                        }
                    },
                }
            },
        }
        article["versioncreated"] = datetime.datetime(
            year=2017, month=2, day=24, hour=16, minute=40, second=56, tzinfo=utc
        )
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        item = json.loads(doc)
        self.assertEqual(item["renditions"]["viewImage"]["media"], "5c11ece81d41c89113ed202b")



from bson import ObjectId
from datetime import datetime, timedelta
from superdesk.tests import TestCase
from superdesk.audit import PurgeAudit
from superdesk import get_resource_service


class AuditTestCase(TestCase):
    def testAuditPurge(self):
        now = datetime.utcnow()
        self.app.data.insert("archive", [{"_id": 2}])
        self.app.data.insert(
            "audit",
            [
                {"_id": ObjectId.from_datetime(now - timedelta(minutes=50)), "resource": "user"},
                {
                    "_id": ObjectId.from_datetime(now - timedelta(minutes=30)),
                    "resource": "archive",
                    "extra": {"guid": 2},
                    "audit_id": 2,
                },
                {
                    "_id": ObjectId.from_datetime(now - timedelta(minutes=10)),
                    "resource": "archive",
                    "extra": {"guid": 3},
                    "audit_id": 3,
                },
                {"_id": ObjectId(), "resource": "archive_autosave", "audit_id": 4},
            ],
        )
        self.app.config["AUDIT_EXPIRY_MINUTES"] = 5
        PurgeAudit().run()
        self.assertEqual(get_resource_service("audit").find({}).count(), 1)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import hashlib
import datetime
import re
from superdesk.utc import utc
import json
from copy import deepcopy
from superdesk import etree as sd_etree
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import FeedParser
from superdesk.io.iptc import subject_codes
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, GUID_TAG, Priority
from superdesk.metadata.utils import generate_guid
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, ITEM_URGENCY, ITEM_PRIORITY
from apps.archive.common import format_dateline_to_locmmmddsrc
from superdesk.utc import get_date
from flask import current_app as app
from superdesk import get_resource_service


logger = logging.getLogger(__name__)


class APMediaFeedParser(FeedParser):

    NAME = "ap_media"

    label = "AP Media API"

    direct_copy_properties = (
        "type",
        ITEM_URGENCY,
        "uri",
        "language",
        "pubstatus",
        "ednote",
        "headline",
        "slugline",
        "copyrightnotice",
    )

    # Mapping the received urgensy field to a priority value
    priority_map = {
        1: Priority.Flash.value,  # Flash
        2: Priority.Three_Paragraph.value,  # Bulletin
        3: Priority.Urgent.value,  # Urgent
        4: Priority.Ordinary.value,  # Routine
        5: Priority.Ordinary.value,  # Daily
        6: Priority.Ordinary.value,  # Release at will
        7: Priority.Ordinary.value,  # Weekday advance
        8: Priority.Ordinary.value,
    }  # Weekend advance

    slug_code_map = {  # Basketball
        "BKC-": "15008000",
        "BKL-": "15008000",
        "BKN-": "15008000",
        "BKW-": "15008000",
        "BKO-": "15008000",
        # Baseball
        "BBA-": "15007000",
        "BBC-": "15007000",
        "BBM-": "15007000",
        "BBN-": "15007000",
        "BBO-": "15007000",
        "BBY-": "15007000",
        # Boxing
        "BOX-": "15014000",
        # Motor racing
        "CAR-": "15039000",
        # Cycling
        "CYC-": "15019000",
        # American Football
        "FBL-": "15003000",
        "FBN-": "15003000",
        "FBO-": "15003000",
        # Figure skating
        "FIG-": "15025000",
        # Golf
        "GLF-": "15027000",
        # Ice Hockey
        "HKN-": "15031000",
        "HKO-": "15031000",
        # Horse Racing
        "RAC-": "15030000",
        # Soccer
        "SOC-": "15054000",
        # Tennis
        "TEN-": "15065000",
        # Cricket
        "CRI-": "15017000",
        # Rugby league
        "RGL-": "15048000",
    }

    RELATED_FIELD = None

    RENDITIONS_MAPPING = {
        "baseImage": "main",
    }

    def __init__(self):
        super().__init__()

    def can_parse(self, s_json):
        try:
            item = s_json.get("data", {}).get("item") and s_json.get("api_version")

            if item is None:
                return False

            return True
        except Exception:
            pass

        return False

    def datetime(self, string):
        try:
            return datetime.datetime.strptime(string, "%Y-%m-%dT%H:%M:%S+0000")
        except ValueError:
            return datetime.datetime.strptime(string, "%Y-%m-%dT%H:%M:%SZ").replace(tzinfo=utc)

    def categorisation_mapping(self, in_item, item):
        """
        Function that maps categorisation fields may be overloaded if not required
        :param in_item:
        :param item:
        :return:
        """
        if in_item.get("subject"):
            categories = [{"qcode": c.get("code")} for c in in_item.get("subject") if c.get("rels") == ["category"]]
            if len(categories):
                item["anpa_category"] = [categories[0]]
                self._map_category_codes(item)

        self._map_sluglines_to_subjects(item=item)

    def _map_category_codes(self, item):
        """Map the category code that has been received to a more palatable value

        :param item:
        :return:
        """
        category_code_map = get_resource_service("vocabularies").find_one(req=None, _id="ap_category_map")
        if category_code_map:
            map = {c["ap_code"]: c["category_code"] for c in category_code_map["items"] if c["is_active"]}
            for category in item.get("anpa_category", []):
                category["qcode"] = map.get(category.get("qcode").lower(), map.get("default"))

    def _map_sluglines_to_subjects(self, item):
        """The first few characters of the slugline may match AP supplimetal categories this is used to set the subject code.

        :param item:
        :return:
        """
        if len(item.get("slugline", "")) > 4:
            qcode = self.slug_code_map.get(item["slugline"][:4])
            if qcode:
                try:
                    item["subject"] = []
                    item["subject"].append({"qcode": qcode, "name": subject_codes[qcode]})
                except KeyError:
                    logger.debug("Subject code '%s' not found" % qcode)

    def parse(self, s_json, provider=None):
        in_item = s_json.get("data", {}).get("item")
        nitf_item = s_json.get("nitf", {})
        item = {"guid": in_item.get("altids", {}).get("itemid") + ":" + str(in_item.get("version"))}
        item["source"] = provider.get("source") if provider else "AP"

        for copy_property in self.direct_copy_properties:
            if in_item.get(copy_property) is not None:
                item[copy_property] = in_item[copy_property]

        if in_item.get("version"):
            item["version"] = in_item["version"]

        if in_item.get("versioncreated"):
            item["versioncreated"] = self.datetime(in_item.get("versioncreated"))

        if in_item.get("firstcreated"):
            item["firstcreated"] = self.datetime(in_item.get("firstcreated"))

        if len(in_item.get("infosource", [])):
            item["original_source"] = ",".join([n.get("name") for n in in_item.get("infosource", [])])

        if in_item.get("datelinelocation"):
            cities = app.locators.find_cities()
            # Try to find a single matching city either by city and country or city country and state
            located = [
                c
                for c in cities
                if c["city"] == in_item.get("datelinelocation").get("city")
                and c["country"] == in_item.get("datelinelocation").get("countryname")
            ]
            if len(located) > 1:
                located = [
                    c
                    for c in cities
                    if c["city"] == in_item.get("datelinelocation").get("city")
                    and c["country"] == in_item.get("datelinelocation").get("countryname")
                    and c["state"] == in_item.get("datelinelocation").get("countryareaname")
                ]
            if len(located) == 1:
                item["dateline"] = dict()
                item["dateline"]["located"] = located[0]
                item["dateline"]["source"] = provider.get("source")
                item["dateline"]["text"] = format_dateline_to_locmmmddsrc(
                    item["dateline"]["located"], get_date(item["firstcreated"]), provider.get("source")
                )

        if len(in_item.get("bylines", [])):
            item["byline"] = ",".join(
                [
                    n.get("name")
                    if n.get("name")
                    else n.get("by", "") + (" ({})".format(n.get("title")) if n.get("title") else "")
                    for n in in_item.get("bylines", [])
                ]
            )
            if item.get("byline").startswith("By "):
                item["byline"] = item["byline"][3:]

        if len(in_item.get("usageterms", [])):
            item["usageterms"] = ", ".join([n for n in in_item.get("usageterms", [])])

        if in_item.get("type") == "picture":
            if in_item.get("renditions"):
                self._parse_renditions(in_item["renditions"], item, provider)

            if in_item.get("description_caption"):
                item["description_text"] = in_item.get("description_caption")
                item["archive_description"] = in_item.get("description_caption")

            if in_item.get("description_creditline"):
                item["credit"] = in_item.get("description_creditline")

            if in_item.get("photographer", {}).get("name"):
                item["byline"] = in_item.get("photographer", {}).get("name")

        if in_item.get("type") == "text":
            # Peel off the take key if possible
            if "," in item["slugline"]:
                item["anpa_take_key"] = item["slugline"].split(",")[1]
                item["slugline"] = item["slugline"].split(",")[0]
            if item["slugline"].startswith("BC-"):
                item["slugline"] = item["slugline"][3:]
            if item.get("ednote", "").startswith("Eds:"):
                item["ednote"] = item["ednote"][5:]
            if in_item.get("headline_extended"):
                item["abstract"] = in_item.get("headline_extended")

            self.categorisation_mapping(in_item, item)

            # Map the urgency to urgency and priority
            if in_item.get("urgency"):
                item[ITEM_URGENCY] = int(in_item["urgency"]) if in_item["urgency"] <= 5 else 5
                item[ITEM_PRIORITY] = self.priority_map.get(in_item["urgency"], 5)

            if nitf_item.get("body_html"):
                # item['body_html'] = sd_etree.clean_html_str(nitf_item.get('body_html'))
                item["body_html"] = nitf_item.get("body_html").replace('<block id="Main">', "").replace("</block>", "")

        if s_json.get("associations"):
            self._parse_associations(s_json["associations"], item, provider)

        return item

    def _parse_associations(self, associations, item, provider=None):
        related_id = getattr(self, "RELATED_ID", app.config.get("INGEST_AP_RELATED_ID"))
        if related_id:
            item["associations"] = {}
            for key, raw in associations.items():
                item["associations"]["{}--{}".format(related_id, key)] = self.parse(raw, provider)

    def _parse_renditions(self, renditions, item, provider=None):
        try:
            item["renditions"] = {}
            for dest, src in self.RENDITIONS_MAPPING.items():
                rend = renditions[src]
                item["renditions"][dest] = {
                    "href": rend["href"],
                    "mimetype": rend["mimetype"],
                    "width": rend["width"],
                    "height": rend["height"],
                }
        except KeyError:
            pass


register_feed_parser(APMediaFeedParser.NAME, APMediaFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from unittest import mock
from unittest.mock import MagicMock, patch

from superdesk.tests import TestCase


class FakeSuperdesk:
    def __init__(self):
        self.services = {}  # can be accessed and overriden by test methods

    def get_resource_service(self, service_name):
        return self.services.get(service_name)


fake_superdesk = FakeSuperdesk()


@mock.patch("superdesk.io.commands.update_ingest.superdesk", fake_superdesk)
class GetProviderRoutingSchemeTestCase(TestCase):
    """Tests for the get_provider_routing_scheme() function."""

    def setUp(self):
        try:
            from superdesk.io.commands.update_ingest import get_provider_routing_scheme
        except ImportError:
            self.fail("Could not import function under test " + "(get_provider_routing_scheme).")
        else:
            self.funcToTest = get_provider_routing_scheme

        fake_superdesk.services = {
            "routing_schemes": MagicMock(name="routing_schemes"),
            "content_filters": MagicMock(name="content_filters"),
        }

    def test_returns_none_if_no_provider_scheme_defined(self):
        fake_provider = {"routing_scheme": None}
        result = self.funcToTest(fake_provider)
        self.assertIsNone(result)

    def test_returns_scheme_config_from_db_if_scheme_defined(self):
        fake_scheme = {"_id": "abc123", "rules": []}
        schemes_service = fake_superdesk.services["routing_schemes"]
        schemes_service.find_one.return_value = fake_scheme

        fake_provider = {"routing_scheme": "abc123"}
        result = self.funcToTest(fake_provider)

        # check that correct scheme has been fetched and returned
        self.assertTrue(schemes_service.find_one.called)
        args, kwargs = schemes_service.find_one.call_args
        self.assertEqual(kwargs.get("_id"), "abc123")
        self.assertEqual(result, fake_scheme)

    def test_includes_content_filters_in_returned_scheme(self):
        fake_scheme = {
            "_id": "abc123",
            "rules": [
                {"filter": "filter_id_4"},
                {"filter": "filter_id_8"},
            ],
        }
        schemes_service = fake_superdesk.services["routing_schemes"]
        schemes_service.find_one.return_value = fake_scheme

        filters_service = fake_superdesk.services["content_filters"]
        filters_service.find_one.side_effect = [
            {"_id": "filter_id_4"},
            {"_id": "filter_id_8"},
        ]

        fake_provider = {"routing_scheme": "abc123"}
        result = self.funcToTest(fake_provider)

        scheme_rules = result.get("rules", [])
        self.assertEqual(len(scheme_rules), 2)
        self.assertEqual(scheme_rules[0].get("filter"), {"_id": "filter_id_4"})
        self.assertEqual(scheme_rules[1].get("filter"), {"_id": "filter_id_8"})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk import get_backend
from apps.products.resource import ProductsResource
from apps.products.service import ProductsService
from apps.products.product_test import ProductTestService, ProductTestResource


def init_app(app):
    endpoint_name = "products"
    service = ProductsService(endpoint_name, backend=get_backend())
    ProductsResource(endpoint_name, app=app, service=service)

    endpoint_name = "product_tests"
    service = ProductTestService(endpoint_name, backend=superdesk.get_backend())
    ProductTestResource(endpoint_name, app=app, service=service)

    superdesk.privilege(name="products", label="Products Management", description="User can manage product lists.")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from builtins import NotImplementedError


class DataLayer:
    def etag(self, doc):
        raise NotImplementedError()

    def find_one(self, resource, filter, projection, options):
        raise NotImplementedError()

    def find(self, resource, filter, projection, options):
        raise NotImplementedError()

    def create(self, resource, docs):
        raise NotImplementedError()

    def update(self, resource, filter, doc):
        raise NotImplementedError()

    def replace(self, resource, filter, doc):
        raise NotImplementedError()

    def delete(self, resource, filter):
        raise NotImplementedError()



import unittest

from superdesk.io.feeding_services.http_base_service import HTTPFeedingServiceBase


class FeedingServiceWithUrl(HTTPFeedingServiceBase):

    URL = "http://example.com"

    fields = []

    def _update(self, provider, update):
        pass


class TestFeedingService(unittest.TestCase):
    def test_validate_config_url_null(self):
        service = FeedingServiceWithUrl()
        service.provider = {"config": {"url": None}}
        service.validate_config()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import arrow
import datetime
import pytz
from pytz import utc, timezone  # flake8: noqa

tzinfo = getattr(datetime, "tzinfo", object)


def utcnow():
    """Get tz aware datetime object.

    Remove microseconds which can't be persisted by mongo so we have
    the values consistent in both mongo and elastic.
    """
    if hasattr(datetime.datetime, "now"):
        now = datetime.datetime.now(tz=utc)
    else:
        now = datetime.datetime.utcnow()
    return now.replace(microsecond=0)


def get_date(date_or_string):
    if date_or_string:
        return arrow.get(date_or_string).datetime


def get_expiry_date(minutes, offset=None):
    if minutes is None or minutes <= 0:
        return None
    if offset:
        if type(offset) is datetime.datetime:
            try:
                return offset + datetime.timedelta(minutes=minutes)
            except OverflowError:
                return
        else:
            raise TypeError("offset must be a datetime.date, not a %s" % type(offset))
    else:
        try:
            return utcnow() + datetime.timedelta(minutes=minutes)
        except OverflowError:  # very big number, never expire
            return


def local_to_utc(local_tz_name, local_datetime):
    """
    Converts the local_datetime to utc
    :param local_tz_name: Name of the local timezone
    :param local_datetime: Value of the local datetime
    :return: the utc datetime
    """
    if local_datetime:
        local_tz = pytz.timezone(local_tz_name)
        utc_dat = local_tz.localize(local_datetime.replace(tzinfo=None))
        return pytz.utc.normalize(utc_dat)


def utc_to_local(local_tz_name, utc_datetime):
    """
    Converts utc datetime to local
    :param local_tz_name: Name of the local timezone
    :param utc_datetime: Value of the utc datetime
    :return: local datetime
    """
    if utc_datetime and local_tz_name:
        if not utc_datetime.tzinfo:
            utc_datetime = utc_datetime.replace(tzinfo=pytz.utc)
        local_tz = pytz.timezone(local_tz_name)
        local_dt = utc_datetime.astimezone(local_tz)
        return local_tz.normalize(local_dt)


def set_time(current_datetime, timestr):
    """Set time of given datetime according to timestr.

    Time format for timestr is `%H:%M:%S`, eg. 10:14:00.

    :param datetime current_datetime
    :param string timestr
    :param int second
    """
    if timestr is None:
        timestr = "00:00:00"
    time = datetime.datetime.strptime(timestr, "%H:%M:%S")
    return current_datetime.replace(hour=time.hour, minute=time.minute, second=time.second)


def get_timezone_offset(local_tz_name, utc_datetime):
    """
    Get the timezone offset
    :param string local_tz_name:
    :param datetime utc_datetime:
    :return string utc offset
    """
    try:
        local_dt = utc_to_local(local_tz_name, utc_datetime)
        return local_dt.strftime("%z")
    except Exception:
        return utcnow().strftime("%z")


def query_datetime(datetime_value, query):
    """Checks the datetime_value against the query provided.

    The query format is similar to that of MongoDB BSON comparison operators.
    It uses `$eq`, `$gt`, `$gte`, `$lt`, `$lte` and `$ne`. Combine these operators together in a dictionary
    to provide the datetime checking functionality. This is currently used when finding files from Amazon S3, but
    could possibly be used in other areas.

    :param datetime.datetime datetime_value: The datetime value used to check against the query
    :param dict query: The query parameters used to check against the datetime_value
    :return boolean: True if all comparison operators pass, else False
    """
    if "$lte" in query and datetime_value > query["$lte"]:
        return False
    elif "$lt" in query and datetime_value >= query["$lt"]:
        return False
    elif "$gte" in query and datetime_value < query["$gte"]:
        return False
    elif "$gt" in query and datetime_value <= query["$gt"]:
        return False
    elif "$eq" in query and datetime_value != query["$eq"]:
        return False
    elif "$ne" in query and datetime_value == query["$ne"]:
        return False
    return True



from ..service import ProdApiService


class UsersService(ProdApiService):
    excluded_fields = {
        "user_preferences",
        "session_preferences",
        "password",
    } | ProdApiService.excluded_fields



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os

from superdesk.io.feed_parsers.iptc7901 import IPTC7901FeedParser
from superdesk.tests import TestCase


def fixture(filename):
    dirname = os.path.dirname(os.path.realpath(__file__))
    return os.path.normpath(os.path.join(dirname, "../fixtures", filename))


class IptcTestCase(TestCase):

    parser = IPTC7901FeedParser()

    def open(self, filename):
        provider = {"name": "Test"}
        return self.parser.parse(fixture(filename), provider)

    def test_open_iptc7901_file(self):
        with self.app.app_context():
            item = self.open("IPTC7901.txt")
            self.assertEqual("text", item["type"])
            self.assertEqual("062", item["ingest_provider_sequence"])
            self.assertEqual("i", item["anpa_category"][0]["qcode"])
            self.assertEqual(211, item["word_count"])
            self.assertEqual("Germany Social Democrats: Coalition talks with Merkel could fail =", item["headline"])
            self.assertRegex(item["body_html"], "^\n   Berlin")
            self.assertEqual("Germany-politics", item["slugline"])
            self.assertEqual(4, item["priority"])
            self.assertEqual([{"qcode": "i"}], item["anpa_category"])
            self.assertTrue(item["ednote"].find("## Editorial contacts"))

    def test_open_iptc7901_file_odd_charset(self):
        with self.app.app_context():
            item = self.open("IPTC7901_odd_charset.txt")
            self.assertTrue(item["body_html"].find("Müller"))
            self.assertTrue(item["ednote"].find("## Editorial contacts"))

    def test_map_priority(self):
        self.assertEqual(1, self.parser.map_priority("1"))
        self.assertEqual(2, self.parser.map_priority("2"))
        self.assertEqual(3, self.parser.map_priority("3"))
        self.assertEqual(5, self.parser.map_priority("5"))
        self.assertEqual(6, self.parser.map_priority("eee"))
        self.assertEqual(6, self.parser.map_priority(None))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from eve.utils import config
from superdesk.users import UsersService, UsersResource, is_admin  # NOQA


logger = logging.getLogger(__name__)


class ADUsersService(UsersService):
    """
    Service class for UsersResource and should be used when AD is active.
    """

    readonly_fields = ["email", "first_name", "last_name"]

    def on_fetched(self, doc):
        super().on_fetched(doc)
        for document in doc["_items"]:
            self.set_defaults(document)

    def on_fetched_item(self, doc):
        super().on_fetched_item(doc)
        self.set_defaults(doc)

    def set_defaults(self, doc):
        """Set the readonly fields for LDAP user.

        :param dict doc: user
        """
        readonly = {}
        user_attributes = config.LDAP_USER_ATTRIBUTES
        for value in user_attributes.values():
            if value in self.readonly_fields:
                readonly[value] = True

        doc["_readonly"] = readonly



import logging

from typing import List

from . import SuperdeskMediaStorage
from .desk_media_storage import SuperdeskGridFSMediaStorage
from .amazon_media_storage import AmazonMediaStorage


logger = logging.getLogger(__name__)


def log_missing_media(id_or_filename, resource=None):
    logger.error(
        "Media item not found",
        extra=dict(
            id=id_or_filename,
            resource=resource,
        ),
    )


class MissingMediaError(ValueError):
    pass


class ProxyMediaStorage(SuperdeskMediaStorage):

    _storage: List[SuperdeskMediaStorage]

    def __init__(self, app):
        super().__init__(app)

        self._storage = [SuperdeskGridFSMediaStorage(app)]

        if app.config.get("AMAZON_CONTAINER_NAME"):
            # make amazon first if configured, so it will be the default
            self._storage.insert(0, AmazonMediaStorage(app))

    def storage(self, id_or_filename=None, resource=None, fallback=False) -> SuperdeskMediaStorage:
        if id_or_filename:
            for storage in self._storage:
                if storage.exists(id_or_filename, resource):
                    logger.debug("got media from storage id=%s storage=%s", id_or_filename, storage)
                    return storage
            if not fallback:
                log_missing_media(id_or_filename, resource)
                raise MissingMediaError

        return self._storage[0]

    def get(self, id_or_filename, resource=None):
        try:
            return self.storage(id_or_filename, resource).get(id_or_filename, resource=resource)
        except MissingMediaError:
            return

    def delete(self, id_or_filename, resource=None):
        try:
            return self.storage(id_or_filename, resource).delete(id_or_filename, resource=resource)
        except MissingMediaError:
            return True

    def exists(self, id_or_filename, resource=None):
        try:
            return self.storage(id_or_filename, resource).exists(id_or_filename, resource=resource)
        except MissingMediaError:
            return False

    def put(self, content, filename=None, content_type=None, metadata=None, resource=None, **kwargs):
        return self.storage(None, resource).put(
            content, filename=filename, content_type=content_type, metadata=metadata, resource=resource, **kwargs
        )

    def url_for_media(self, media_id, content_type=None):
        return self.storage(media_id, fallback=True).url_for_media(media_id, content_type=content_type)

    def url_for_download(self, media_id, content_type=None):
        return self.storage(media_id, fallback=True).url_for_download(media_id, content_type=content_type)

    def fetch_rendition(self, rendition):
        if rendition.get("media"):
            return self.get(rendition["media"])

        for storage in self._storage:
            media = storage.fetch_rendition(rendition)
            if media:
                return media

    def get_by_filename(self, filename):
        for storage in self._storage:
            media = storage.get_by_filename(filename)
            if media:
                return media



"""Here you can import/implement macros.

This module will get reloaded per request to be up to date.

Use `superdesk.macro_register.macros.register` for registration.
"""

import os
import sys
import imp
import logging
import importlib
import logging


logger = logging.getLogger(__name__)


logger = logging.getLogger(__name__)

macro_replacement_fields = {"body_html", "body_text", "abstract", "headline", "slugline", "description_text"}


def load_macros(path, package_prefix="superdesk.macros"):
    """Load macros from given path

    :param str path:
    :param str package_prefix:
    """
    macros = [
        f[:-3] for f in os.listdir(path) if f.endswith(".py") and not f.endswith("_test.py") and not f.startswith("__")
    ]

    for macro in macros:
        module = "{}.{}".format(package_prefix, macro)
        try:
            if module in sys.modules.keys():
                m = sys.modules[module]
                imp.reload(m)
            else:
                importlib.import_module(module)
        except Exception as e:
            logger.warning("Can't import macro {module}: {reason}".format(module=module, reason=e))


def init_app(app):
    load_macros(os.path.dirname(__file__), __name__)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import io
import re
import logging
import collections

from flask import json, current_app as app, request
from simplejson.errors import JSONDecodeError
from eve.utils import config

from superdesk.errors import SuperdeskApiError
from superdesk.services import BaseService
from superdesk.notification import push_notification
from apps.dictionaries.resource import DICTIONARY_FILE, DictionaryType
from flask_babel import _


FILE_ID = "_file_id"


logger = logging.getLogger(__name__)


def encode_dict(words_dict):
    return json.dumps(words_dict)


def decode_dict(words_list):
    if isinstance(words_list, dict):
        return words_list
    return json.loads(words_list)


def train(features):
    model = collections.defaultdict(lambda: 1)
    for f in features:
        model[f] += 1
    return model


def words(text):
    return [w.strip() for w in re.findall("[^\n]+", text) if not w.isdigit()]


def add_word(words, word, count):
    """Add word with count to words model.

    :param words: words model
    :param word
    :param count
    """
    words.setdefault(word, 0)
    try:
        words[word] += count
    except TypeError as e:
        logger.error(word)
        logger.error(count)
        raise e


def add_words(nwords, text, val=1):
    for word in words(text):
        add_word(nwords, word, val)


def read(stream):
    return stream.read().decode("utf-8").replace("\ufeff", "")


def merge(doc, words):
    doc.setdefault("content", {})
    for word, count in words.items():
        add_word(doc["content"], word, count)


def fetch_dict(doc):
    """Fetch dictionary content after storing it, from gridfs or json.

    :param doc
    """
    if doc and doc.get(FILE_ID):
        content_file = app.storage.get(doc[FILE_ID])
        content = json.loads(content_file.read())
        return content

    if doc and doc.get("content"):
        return decode_dict(doc["content"])

    return {}


def store_dict(updates, original):
    """Store dictionary content.

    In case it's too big - use gridfs, otherwise just json encode it.

    :param updates
    :param original
    """
    content = updates.pop("content", {})
    if content:
        content_json = json.dumps(content)
        if is_big(content_json):
            content_binary = io.BytesIO(content_json.encode("utf-8"))
            updates[FILE_ID] = app.storage.put(content_binary)
            updates["content"] = None
        else:
            updates["content"] = content_json
            updates[FILE_ID] = None

    if original.get(FILE_ID):
        app.storage.delete(original[FILE_ID])


def is_big(json_data):
    """Test if given json data is too big for mongo document.

    Using 1MB as limit atm.

    :param json_data
    """
    return len(json_data) > 1000000


def read_from_file(doc):
    """Read from plain text file

    One word per line
    UTF-8 encoding
    """
    content = doc.pop(DICTIONARY_FILE)
    if "text/" not in content.mimetype:
        raise SuperdeskApiError.badRequestError(_("A text dictionary file is required"))
    return train(words(read(content)))


class DictionaryService(BaseService):
    def on_create(self, docs):
        if len(docs) == 1 and "content" in docs[0]:
            # works around Eve behaviour which creates sub-dict on each "." it finds in keys
            # cf. SDESK-3083
            # We also test request.data because it is not set in behave tests, and they would fail without it
            try:
                docs[0]["content"] = json.loads(request.data.decode("utf-8"))["content"]
            except (KeyError, JSONDecodeError, RuntimeError):
                # request.data is not set during tests, so we ignore those errors
                pass

        for doc in docs:
            if self.is_duplicate_dictionary(doc):
                raise SuperdeskApiError.badRequestError(
                    message=_("The dictionary already exists"), payload={"name": "duplicate"}
                )

            self.__set_default(doc)
            self._validate_dictionary(doc)

            if doc.get(DICTIONARY_FILE):
                words = read_from_file(doc)
                merge(doc, words)

            store_dict(doc, {})

    def on_created(self, docs):
        for doc in docs:
            push_notification("dictionary:created", language=doc.get("language_id"))

    def find_one(self, req, **lookup):
        doc = super().find_one(req, **lookup)
        if doc:
            doc["content"] = fetch_dict(doc)
        return doc

    def _validate_dictionary(self, updates, original=None):
        if original is None:
            original = {}
        dict_type = updates.get("type", original.get("type", DictionaryType.DICTIONARY.value))
        if dict_type == DictionaryType.ABBREVIATIONS.value and not updates.get("user", original.get("user")):
            raise SuperdeskApiError.badRequestError(
                message=_("User is required for the abbreviations dictionary."), payload={"user": "missing"}
            )

        if original and dict_type != original.get("type", DictionaryType.DICTIONARY.value):
            raise SuperdeskApiError.badRequestError(message=_("The dictionary type cannot be changed."))

    def get_base_language(self, lang):
        if lang and lang.find("-") > 0:
            return lang.split("-")[0]

    def is_duplicate_dictionary(self, doc):
        return self.find_one(
            req=None,
            name=doc["name"],
            language_id=doc["language_id"],
            type=doc.get("type", DictionaryType.DICTIONARY.value),
        )

    def get_dictionaries(self, lang):
        """Returns all the active dictionaries.

        If both the language (en-AU)
        and the base language (en) available, it will return the dict with language

        :param lang:
        :return:
        """
        languages = [{"language_id": lang}]
        base_language = self.get_base_language(lang)
        if base_language:
            languages.append({"language_id": base_language})

        lookup = {
            "$and": [
                {"$or": languages},
                {"$or": [{"is_active": {"$exists": 0}}, {"is_active": "true"}]},
                {"$or": [{"type": {"$exists": 0}}, {"type": DictionaryType.DICTIONARY.value}]},
            ]
        }
        dicts = list(self.get(req=None, lookup=lookup))
        langs = [d["language_id"] for d in dicts]

        if base_language and base_language in langs and lang in langs:
            dicts = [d for d in dicts if d["language_id"] != base_language]

        return dicts

    def get_model_for_lang(self, lang):
        """Get model for given language.

        It will use all active dictionaries for given language combined.

        :param lang: language code
        """
        model = {}
        dicts = self.get_dictionaries(lang)

        for _dict in dicts:
            content = fetch_dict(_dict)
            for word, count in content.items():
                add_word(model, word, count)
        return model

    def on_update(self, updates, original):
        if "content" in updates:
            # works around Eve behaviour which creates sub-dict on each "." it finds in keys
            # cf. SDESK-3083
            try:
                updates["content"] = json.loads(request.data.decode("utf-8"))["content"]
            except (KeyError, JSONDecodeError, RuntimeError):
                # request.data is not set during tests, so we ignore those errors
                pass

        user = original.get("user")
        name = updates.get("name")
        language_id = updates.get("language_id")

        # if user is present it means dictonary is personal/abbrevations.
        if user and language_id:
            personal_dictionary = {
                "name": str(user) + ":" + language_id,
                "language_id": language_id,
                "type": original.get("type"),
            }

        if (
            user
            and language_id
            and language_id != original.get("language_id")
            and self.is_duplicate_dictionary(personal_dictionary)
        ):
            raise SuperdeskApiError.badRequestError(
                message=_("The dictionary already exists"), payload={"name": "duplicate"}
            )
        elif name and name != original.get("name") and self.find_one(req=None, name=name):
            raise SuperdeskApiError.badRequestError(
                message=_("The dictionary already exists"), payload={"name": "duplicate"}
            )

        # parse json list
        if updates.get("content_list"):
            updates["content"] = json.loads(updates.pop("content_list"))

        if "type" not in original:
            self.__set_default(updates)

        self._validate_dictionary(updates, original)

        # handle manual changes
        if original.get("type", DictionaryType.DICTIONARY.value) == DictionaryType.DICTIONARY.value:
            nwords = fetch_dict(original).copy()
            for word, val in updates.get("content", {}).items():
                if val:
                    add_words(nwords, word, val)
                else:
                    nwords.pop(word, None)

            updates["content"] = nwords

        # handle uploaded file
        if updates.get(DICTIONARY_FILE):
            file_words = read_from_file(updates)
            merge(updates, file_words)

        store_dict(updates, original)

    def on_updated(self, updates, original):
        push_notification("dictionary:updated", language=updates.get("language_id", original.get("language_id")))

    def __set_default(self, doc):
        if "type" not in doc:
            doc["type"] = DictionaryType.DICTIONARY.value

    def on_fetched_item(self, doc):
        self.__enhance_items([doc])

    def on_fetched(self, docs):
        self.__enhance_items(docs[config.ITEMS])

    def __enhance_items(self, docs):
        for doc in docs:
            self.__set_default(doc)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime
import logging

from bson.objectid import ObjectId
from flask import g
from flask_babel import _

import superdesk
from superdesk import get_resource_service
from superdesk.emails import send_activity_emails
from superdesk.errors import SuperdeskApiError, add_notifier
from superdesk.notification import push_notification
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.utc import utcnow
from eve.utils import ParsedRequest
from superdesk.metadata.item import PUBLISH_STATES
import json

log = logging.getLogger(__name__)


def init_app(app):

    endpoint_name = "activity"
    service = ActivityService(endpoint_name, backend=superdesk.get_backend())
    ActivityResource(endpoint_name, app=app, service=service)

    # Registering with intrinsic privileges because: A user should be able to mark as read their own notifications.
    superdesk.intrinsic_privilege(resource_name="activity", method=["PATCH"])


class ActivityResource(Resource):
    endpoint_name = "activity"
    resource_methods = ["GET"]
    item_methods = ["GET", "PATCH"]
    schema = {
        "name": {"type": "string"},
        "message": {"type": "string"},
        "data": {"type": "dict", "schema": {}, "allow_unknown": True},
        "recipients": {
            "type": "list",
            "schema": {
                "type": "dict",
                "schema": {
                    "user_id": Resource.rel("users"),
                    "read": {"type": "boolean", "default": False},
                    "desk_id": Resource.rel("desks"),
                },
            },
        },
        "item": Resource.rel("archive", type="string"),
        "item_slugline": {"type": "string"},
        "user": Resource.rel("users"),
        "user_name": {"type": "string"},
        "desk": Resource.rel("desks"),
        "resource": {"type": "string"},
    }
    exclude = {endpoint_name, "notification"}
    datasource = {
        "default_sort": [("_created", -1)],
        "filter": {"_created": {"$gte": utcnow() - datetime.timedelta(days=1)}},
    }
    superdesk.register_default_user_preference(
        "email:notification",
        {
            "type": "bool",
            "enabled": True,
            "default": True,
            "label": _("Send notifications via email"),
            "category": _("notifications"),
        },
    )
    superdesk.register_default_user_preference(
        "desktop:notification",
        {
            "type": "bool",
            "enabled": True,
            "default": False,
            "label": _("Allow Desktop Notifications"),
            "category": _("notifications"),
        },
    )


class ActivityService(BaseService):
    def get(self, req, lookup):
        """Filter out personal activity on personal items if inquired by another user."""
        if req is None:
            req = ParsedRequest()
        user = getattr(g, "user", None)
        if not user:
            raise SuperdeskApiError.notFoundError("Can not determine user")
        where_cond = {}
        if req.where:
            if req.where[0] != "{":
                req.where = "{" + req.where + "}"
            where_cond = json.loads(req.where)
        for_user = where_cond.get("user", str(user.get("_id")))
        if for_user != str(user.get("_id")):
            where_item = {"$and": [{"desk": {"$ne": None}}, {"desk": {"$exists": True}}, {"resource": "archive"}]}
            where_cond["$or"] = [where_item, {"resource": {"$ne": "archive"}}]
            req.where = json.dumps(where_cond)

        return self.backend.get(self.datasource, req=req, lookup=lookup)

    def on_update(self, updates, original):
        """Called on the patch request to mark a activity/notification/comment as read and nothing else

        :param updates:
        :param original:
        :return:
        """
        user = getattr(g, "user", None)
        if not user:
            raise SuperdeskApiError.notFoundError("Can not determine user")
        user_id = user.get("_id")

        # make sure that the user making the read notification is in the notification list
        if not self.is_recipient(updates, user_id):
            raise SuperdeskApiError.forbiddenError("User is not in the notification list")

        # make sure the transition is from not read to read
        if not self.is_read(updates, user_id) and self.is_read(original, user_id):
            raise SuperdeskApiError.forbiddenError("Can not set notification as read")

        # make sure that no other users are being marked as read
        for recipient in updates.get("recipients", []):
            if recipient["user_id"] != user_id:
                if self.is_read(updates, recipient["user_id"]) != self.is_read(original, recipient["user_id"]):
                    raise SuperdeskApiError.forbiddenError("Can not set other users notification as read")

        # make sure that no other fields are being up dated just read and _updated
        if len(updates) != 2:
            raise SuperdeskApiError.forbiddenError("Can not update")

    def is_recipient(self, activity, user_id):
        """
        Checks if the given user is in the list of recipients
        """
        return any(r for r in activity.get("recipients", []) if r["user_id"] == user_id)

    def is_read(self, activity, user_id):
        """
        Returns the read value for the given user
        """
        return next((r["read"] for r in activity.get("recipients", []) if r["user_id"] == user_id), False)


ACTIVITY_CREATE = "create"
ACTIVITY_UPDATE = "update"
ACTIVITY_DELETE = "delete"
ACTIVITY_EVENT = "event"
ACTIVITY_ERROR = "error"


def add_activity(
    activity_name, msg, resource=None, item=None, notify=None, notify_desks=None, can_push_notification=True, **data
):
    """
    Adds an activity into activity log.

    This will became part of current user activity log.
    If there is someone set to be notified it will make it into his notifications box.

    :param activity_name: Name of the activity
    :type activity_name: str
    :param msg: Message to be recorded in the activity log
    :type msg: str
    :param resource: resource name generating this activity
    :type resource: str
    :param item: article instance, if the activity is being recorded against an article, default None
    :type item: dict
    :param notify: user identifiers against whom the activity should be recorded, default None
    :type notify: list
    :param notify_desks: desk identifiers if someone mentions Desk Name in comments widget, default None
    :type notify_desks: list
    :param can_push_notification: flag indicating if a notification should be pushed via WebSocket, default True
    :type can_push_notification: bool
    :param data: kwargs
    :type data: dict
    :return: activity object
    :rtype: dict
    """

    activity = {"name": activity_name, "message": msg, "data": data, "resource": resource}

    user = getattr(g, "user", None)
    if user:
        activity["user"] = user.get("_id")
        activity["user_name"] = user.get("display_name", user.get("username"))

    activity["recipients"] = []

    if notify:
        activity["recipients"] = [{"user_id": ObjectId(_id), "read": False} for _id in notify]

    if notify_desks:
        activity["recipients"].extend([{"desk_id": ObjectId(_id), "read": False} for _id in notify_desks])

    if item:
        if item.get("type") == "text":
            activity["item"] = str(item.get("guid", item.get("_id")))
        else:
            # since guid and _id do not match for the item of type picture, audio and video
            # so sending _id as activity['item'] instead of guid for media items
            # and item_id for published media items as _id or guid does not match _id in archive for media items
            activity["item"] = str(item.get("item_id")) if item.get("state") in PUBLISH_STATES else str(item.get("_id"))

        activity["item_slugline"] = item.get("slugline", item.get("headline")) or item.get("unique_name")
        if item.get("task") and item["task"].get("desk"):
            activity["desk"] = ObjectId(item["task"]["desk"])

    get_resource_service(ActivityResource.endpoint_name).post([activity])

    if can_push_notification:
        push_notification(ActivityResource.endpoint_name, _dest=activity["recipients"], activity=activity)

    return activity


def notify_and_add_activity(activity_name, msg, resource=None, item=None, user_list=None, **data):
    """
    Adds the activity and notify enabled and active users via email.
    """

    add_activity(
        activity_name,
        msg=msg,
        resource=resource,
        item=item,
        notify=[str(user.get("_id")) for user in user_list] if user_list else None,
        **data,
    )

    if activity_name == ACTIVITY_ERROR or user_list:
        recipients = get_recipients(user_list, activity_name)

        if activity_name != ACTIVITY_ERROR:
            current_user = getattr(g, "user", None)
            activity = {
                "name": activity_name,
                "message": current_user.get("display_name") + " " + msg if current_user else msg,
                "data": data,
                "resource": resource,
            }
        else:
            activity = {"name": activity_name, "message": "System " + msg, "data": data, "resource": resource}

        if recipients:
            send_activity_emails(activity=activity, recipients=recipients)


def get_recipients(user_list, activity_name):
    if not user_list and activity_name == ACTIVITY_ERROR:
        user_list = get_resource_service("users").get_users_by_user_type("administrator")

    recipients = [
        user.get("email")
        for user in user_list
        if not user.get("needs_activation", True)
        and user.get("is_enabled", False)
        and user.get("is_active", False)
        and get_resource_service("preferences").email_notification_is_enabled(
            preferences=user.get("user_preferences", {})
        )
    ]

    return recipients


add_notifier(notify_and_add_activity)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.services import BaseService
from flask import g, request
from eve.utils import config


class ApiAuditService(BaseService):
    """
    Service class that provides an auditing service for the retrieval of items, assets and packages via the API
    """

    def audit_item(self, item, id):
        """
        Record the retrieval of an item or asset
        :param item: The item to audit
        :param id: id of the item
        :return:
        """
        subscriber = getattr(g, "user", None)
        # in behave testing we get user (dict)
        if isinstance(subscriber, dict):
            subscriber = subscriber.get(config.ID_FIELD)

        audit = {
            "type": item.get("type", ""),
            "subscriber": subscriber,
            "uri": item.get("uri", None),
            "items_id": id,
            "version": item.get("version", ""),
            "remote_addr": request.remote_addr,
        }
        self.post([audit])



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : MarkLark86
# Creation: 2019-12-10 08:57

from superdesk import get_resource_service
from superdesk.commands.data_updates import DataUpdate as _DataUpdate
import logging
from copy import deepcopy

logger = logging.getLogger(__name__)

COVERAGE_PROVIDERS = "coverage_providers"
CONTACT_TYPE = "contact_type"

CVS = {
    "coverage_providers": {
        "schema": {"name": {"required": True, "type": "string"}, "qcode": {"required": True, "type": "string"}}
    },
    "contact_type": {
        "_id": "contact_type",
        "unique_field": "qcode",
        "type": "manageable",
        "display_name": "Contact Types",
        "selection_type": "do not show",
        "schema": {
            "name": {"required": True, "type": "string"},
            "qcode": {"required": True, "type": "string"},
            "assignable": {"required": False, "type": "bool", "label": "Assignable"},
        },
        "items": [],
    },
}

CONTACT_TYPE_LINK = {"required": False, "type": "string", "link_vocab": "contact_type", "link_field": "qcode"}


class DataUpdate(_DataUpdate):
    """Add 'contact_type' and update 'coverage_providers' CVs

    Refer to https://dev.sourcefabric.org/browse/SDESK-4766 for more information
    """

    resource = "vocabularies"

    def forwards(self, mongodb_collection, mongodb_database):
        self._add_contact_type_to_coverage_providers(mongodb_collection)
        self._add_contact_type_cv(mongodb_collection)

    def backwards(self, mongodb_collection, mongodb_database):
        self._remove_contact_type_from_coverage_providers(mongodb_collection)
        self._remove_contact_type_cv(mongodb_collection)

    def _add_contact_type_to_coverage_providers(self, mongodb_collection):
        coverage_providers = mongodb_collection.find_one({"_id": COVERAGE_PROVIDERS})

        if not coverage_providers:
            logger.info('"{}" CV not found. Not need to continue'.format(COVERAGE_PROVIDERS))
            return

        schema = coverage_providers.get("schema") or deepcopy(CVS[COVERAGE_PROVIDERS]["schema"])

        if not schema.get("contact_type"):
            schema["contact_type"] = CONTACT_TYPE_LINK

        items = coverage_providers.get("items") or []
        for item in items:
            if not item.get("contact_type"):
                item["contact_type"] = ""

        mongodb_collection.update({"_id": COVERAGE_PROVIDERS}, {"$set": {"schema": schema, "items": items}})

    def _remove_contact_type_from_coverage_providers(self, mongodb_collection):
        coverage_providers = mongodb_collection.find_one({"_id": COVERAGE_PROVIDERS})

        if not coverage_providers:
            logger.info('"{}" CV not found. Not need to continue'.format(COVERAGE_PROVIDERS))
            return

        schema = coverage_providers.get("schema") or deepcopy(CVS[COVERAGE_PROVIDERS]["schema"])
        schema.pop("contact_type", None)

        items = coverage_providers.get("items") or []
        for item in items:
            item.pop("contact_type", None)

        mongodb_collection.update({"_id": COVERAGE_PROVIDERS}, {"$set": {"schema": schema, "items": items}})

    def _add_contact_type_cv(self, mongodb_collection):
        if mongodb_collection.find_one({"_id": CONTACT_TYPE}):
            logger.info('"{}" CV already defined. Nothing to do'.format(CONTACT_TYPE))
            return

        get_resource_service(self.resource).post([CVS[CONTACT_TYPE]])

    def _remove_contact_type_cv(self, mongodb_collection):
        if not mongodb_collection.find_one({"_id": CONTACT_TYPE}):
            logger.info('"{}" CV not defined. Nothing to do'.format(CONTACT_TYPE))
            return

        get_resource_service(self.resource).delete_action({"_id": CONTACT_TYPE})



# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os.path
import re
import json
from pathlib import Path
from typing import Optional, Dict
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk import config

try:
    import settings
except ImportError:
    # settings doesn't exist during tests
    settings = None
import logging
from importlib_metadata import version as pkg_version, PackageNotFoundError


logger = logging.getLogger(__name__)

GITHUB_TAG_HREF = "https://github.com/superdesk/{repo}/releases/tag/v{version}"
GITHUB_COMMIT_HREF = "https://github.com/superdesk/{package}/commit/{revision}"
GITHUB_BRANCH_HREF = "https://github.com/superdesk/{package}/tree/{branch}"
PYPI_VERSION_HREF = "https://pypi.org/project/{package}/{version}/"
NPM_VERSION_HREF = "https://www.npmjs.com/package/{package}/v/{version}"

RE_REV = re.compile(r"\+g(?P<revision>[a-f0-9]+)(\.[0-9]+)?")


class BackendMetaResource(Resource):
    pass


class BackendMetaService(BaseService):
    """Service givin metadata on backend itself"""

    @staticmethod
    def find_dir(name: str) -> Optional[Path]:
        """Look for a dir in up to 2 parents directories"""
        # depending of the installation (local git or docker)
        # a dir can be in the same path as settings or in the parent
        # this method try both and return None if dir is not found
        if settings is None:
            return None
        current_path = Path(settings.__file__)
        for i in range(2):
            current_path = current_path.parent
            tested_dir = current_path / name
            if tested_dir.is_dir():
                return tested_dir
        return None

    @staticmethod
    def get_commit_href(package: str, revision: str) -> str:
        """Get URL for a Github commit

        if config.REPO_OVERRIDE is set, it will be used
        """
        try:
            repo_override = config.REPO_OVERRIDE
        except AttributeError:
            # config may not be initialised (during tests or beginning of the session)
            repo_override = {}
        return GITHUB_COMMIT_HREF.format(package=repo_override.get(package, package), revision=revision)

    @classmethod
    def get_superdesk_version(cls) -> Optional[Dict[str, str]]:
        """Get version data for main superdesk package

        because superdesk is not installed as a Python package, this works by looking for GIT
        metadata and retrieving the commit version from there
        """
        git_dir = cls.find_dir(".git")
        if git_dir is not None:
            head_path = git_dir / "HEAD"
            try:
                with head_path.open() as f:
                    head = f.read()
                ref_path = head.split()[1]
                with open(os.path.join(git_dir, ref_path)) as rev_f:
                    revision = rev_f.read()
                return {
                    "name": "superdesk",
                    # we don't have semver for Superdesk itself (only for core)
                    "version": revision,
                    "revision": revision,
                    "href": cls.get_commit_href("superdesk", revision),
                }
            except (IOError, IndexError):
                pass
        return None

    @classmethod
    def get_package_version(cls, package: str) -> Optional[Dict[str, str]]:
        """Get version data for a Python package"""
        try:
            version = pkg_version(package)
        except PackageNotFoundError:
            pass
        except Exception as e:
            logger.error(f"Can't retrieve package version: {e}")
            return None
        else:
            semver = ".".join(version.split(".", 3)[:3])
            data = {
                "name": package,
                "version": version,
                "semver": semver,
            }
            rev_match = RE_REV.search(version)
            if rev_match is not None:
                revision = rev_match.group("revision")
                data["revision"] = revision
                data["href"] = cls.get_commit_href(package, revision)
            elif version == semver:
                data["href"] = PYPI_VERSION_HREF.format(
                    package=package,
                    version=semver,
                )
            return data
        return None

    def complete_nodemod_ref(self, data: Dict[str, str], package: str, repo: Optional[str] = None) -> None:
        """Complete when possible missing data for a node module version"""
        if not repo:
            repo = package
        try:
            version = data["version"]
        except KeyError:
            return
        try:
            commit = version.split("#")[1]
        except IndexError:
            if "href" not in data:
                data["href"] = GITHUB_TAG_HREF.format(repo=repo, version=version)
        else:
            if "href" not in data:
                try:
                    int(version, 16)
                    data["href"] = GITHUB_COMMIT_HREF.format(package=repo, revision=commit)
                except ValueError:
                    data["href"] = GITHUB_BRANCH_HREF.format(package=repo, branch=commit)

    def get_nodemod_version(self, package: str, repo: Optional[str] = None) -> Optional[Dict[str, str]]:
        """Get version data for a Node module"""
        # we get superdesk-client-core version and revision from package.json and package-lock.json
        client_dir = self.find_dir("client")
        if client_dir is not None:
            data = {
                "name": repo or package,
            }
            pkg_path = client_dir / "package.json"
            try:
                with pkg_path.open() as f:
                    pkg = json.load(f)
                data["version"] = pkg["dependencies"][package]
            except (IOError, KeyError):
                pass
            pkg_lock_path = client_dir / "package-lock.json"
            try:
                with pkg_lock_path.open() as f:
                    pkg_lock = json.load(f)
                pkg_ver = pkg_lock["dependencies"][package]["version"]
                if pkg_ver.startswith("github:"):
                    __, data["revision"] = pkg_ver[8:].split("#")
                    data["version"] = data["revision"]
                    data["href"] = self.get_commit_href(repo or package, data["revision"])
                else:
                    data["version"] = data["semver"] = pkg_ver
                    data["href"] = NPM_VERSION_HREF.format(package=package, version=pkg_ver)
            except (IOError, KeyError):
                self.complete_nodemod_ref(data, package, repo)

            if len(data) == 1:
                # if we have only the module name, we don't want to display it
                return None
            return data

        return None

    def on_fetched(self, doc):
        doc["modules"] = [
            mod
            for mod in [
                self.get_superdesk_version(),
                self.get_package_version("superdesk-core"),
                self.get_nodemod_version("superdesk-core", repo="superdesk-client-core"),
                self.get_package_version("superdesk-planning"),
                self.get_package_version("superdesk-analytics"),
                self.get_nodemod_version("superdesk-publisher"),
            ]
            if mod is not None
        ]


# it may be useful to have the version of installed packages in backend logs
for package in ("superdesk-core", "superdesk-planning", "superdesk-analytics", "superdesk-published"):
    v_data = BackendMetaService.get_package_version(package)
    if v_data is not None:
        logger.info(f"version of {package!r}: {v_data['version']}")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk

from superdesk import tests


def names(actions):
    return [a["name"] for a in actions]


class WorkflowTestCase(tests.TestCase):
    def test_status_registry(self):
        superdesk.workflow_state(name="test")
        self.assertIn({"name": "test"}, superdesk.get_workflow_states())
        self.assertIn("test", superdesk.allowed_workflow_states)

    def test_action_registry(self):
        superdesk.workflow_action(name="include", include_states=["include"], privileges=["test-privilege"])

        actions = [action for action in superdesk.get_workflow_actions() if action["name"] == "include"]
        self.assertEqual(1, len(actions))
        self.assertIn("include", actions[0]["include_states"])
        self.assertIn("test-privilege", actions[0]["privileges"])

    def test_is_workflow_action_valid(self):
        superdesk.workflow_action(
            name="test_spike", exclude_states=["spiked", "published", "scheduled", "killed"], privileges=["spike"]
        )

        superdesk.workflow_action(
            name="test_on_hold",
            exclude_states=["spiked", "published", "scheduled", "killed", "on_hold"],
            privileges=["on_hold"],
        )

        self.assertTrue(superdesk.is_workflow_state_transition_valid("test_spike", "in_progress"))
        self.assertFalse(superdesk.is_workflow_state_transition_valid("test_spike", "spiked"))
        self.assertTrue(superdesk.is_workflow_state_transition_valid("test_on_hold", "routed"))
        self.assertTrue(superdesk.is_workflow_state_transition_valid("test_on_hold", "fetched"))
        self.assertFalse(superdesk.is_workflow_state_transition_valid("test_on_hold", "published"))
        self.assertFalse(superdesk.is_workflow_state_transition_valid("test_on_hold", "scheduled"))



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : petr
# Creation: 2019-07-29 10:24

from superdesk.commands.data_updates import BaseDataUpdate


class DataUpdate(BaseDataUpdate):

    resource = "content_types"

    def forwards(self, mongodb_collection, mongodb_database):
        filter_ = {"editor.headline.formatOptions": {"$nin": [None, []]}}
        update = {"$set": {"editor.headline.formatOptions": []}}
        result = mongodb_collection.update_many(filter_, update)
        print("matched={} updated={}".format(result.matched_count, result.modified_count))

    def backwards(self, mongodb_collection, mongodb_database):
        pass  # no need to return back to wrong value



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from content_api import MONGO_PREFIX, ELASTIC_PREFIX
from content_api.items.resource import schema as item_schema
from copy import deepcopy


class PublishResource(Resource):
    """A class defining and configuring the /publish API endpoint."""

    schema = deepcopy(item_schema)

    datasource = {
        "source": "items",
        "search_backend": "elastic",
    }

    mongo_prefix = MONGO_PREFIX
    elastic_prefix = ELASTIC_PREFIX

    item_methods = []
    resource_methods = []



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013-2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import hashlib
import datetime
import re
import json
from copy import deepcopy

from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import FeedParser
from superdesk.io.iptc import subject_codes
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, GUID_TAG
from superdesk.metadata.utils import generate_guid

logger = logging.getLogger(__name__)


class BBCNINJSFeedParser(FeedParser):
    """
    Feed Parser for the BBC's NINJS variant
    """

    # Variant documentation can be found at https://docs.ldrs.org.uk/

    NAME = "bbc_ninjs"

    label = "BBC NINJS Variant Parser"

    direct_copy_properties = (
        "uri",
        "language",
        "headline",
        "urgency",
        "pubstatus",
        "mimetype",
        "body_text",
        "body_html",
        "byline",
        "description_text",
    )

    # A dictionary of known subjects and their nearest IPTC qcode analogs
    subject_trans = {
        "policing": "02003000",
        "fire-and-rescue-services": "11006001",
        "crime": "02001000",
        "transport": "04015000",
        "education": "05000000",
        "planning-permission": "11016007",
        "highways": "04015003",
        "waste-management": "04005007",
        "public-safety": "11006002",
        "social-care": "07011000",
        "trading-standards": "04008015",
        "libraries": "01009000",
        "rubbish-collection": "04005007",
        "recycling": "06009000",
        "council-tax": "11013000",
        "environmental-health": "06005000",
        "leisure": "10010000",
        "litter": "06005000",
        "revenue-collection": "04008019",
        "health": "07000000",
        "dog-fouling": "06000000",
        "housing": "11016002",
        "graffiti": "06000000",
        "fly-posting": "06000000",
        "social-services": "14025004",
    }

    def __init__(self):
        super().__init__()

    def can_parse(self, s_json):
        try:
            item_ident = re.search('"total": *[0-9]+', s_json).group()

            if item_ident is None:
                return False

            results_str = re.search("[0-9]+", item_ident).group()

            return results_str is not None and int(results_str) > 0
        except Exception:
            pass

        return False

    def parse(self, s_json, provider=None):
        parsed = []
        json_items = json.loads(s_json).get("item", [])

        for json_item in json_items:
            parsed.extend(self._parse_item(json_item))

        return parsed

    def _parse_item(self, item):
        items = []

        main = self._parse_main(item)
        items.append(main)

        subjects = main.setdefault("subject", [])
        for subject in item["subject"]:
            try:
                parsed_subject = self._parse_subject(subject)
                if parsed_subject is not None:
                    subjects.append(parsed_subject)
            except Exception as ex:
                logger.exception("Exception parsing subject, {}".format(ex))

        associations = main.setdefault("associations", {})
        for association in item["associations"]:
            try:
                key, parsed_association = self._parse_association(association)
                associations[key] = parsed_association

                # Also create an image item out of the association rendition
                image = self._create_image(association, main)
                items.append(image)
            except Exception as ex:
                logger.exception("Exception parsing association, {}".format(ex))

        # If there's only a single type we don't need to build a composite type
        if len(items) > 1:
            items.append(self._create_package(items, main))

        return items

    def _create_package(self, items, main):
        """Builds a composite package for the entire article
        :param items:
        :param main: The main article body
        :return: A composite item dict
        """
        package = {
            ITEM_TYPE: CONTENT_TYPE.COMPOSITE,
            "guid": main["guid"] + "-package",
            "versioncreated": main["versioncreated"],
            "firstcreated": main["firstcreated"],
            "headline": main["headline"],
            "groups": [
                {
                    "id": "root",
                    "role": "grpRole:NEP",
                    "refs": [{"idRef": "main"}],
                },
                {
                    "id": "main",
                    "role": "main",
                    "refs": [],
                },
            ],
        }

        item_references = package["groups"][1]["refs"]
        item_references.append({"residRef": main["guid"]})

        for item in items:
            if item != main:
                item_references.append({"residRef": item["guid"]})

        return package

    def _create_image(self, association, main):
        """Builds an image item from an association
        :param association: The raw association in BBC's ninjs variant
        :param main: The main article body
        :return: A image item dict
        """
        url = association["renditions"]["original"]["href"]
        guid_hash = hashlib.sha1(url.encode("utf8")).hexdigest()

        item = {
            "guid": generate_guid(type=GUID_TAG, id=guid_hash + "-image"),
            ITEM_TYPE: CONTENT_TYPE.PICTURE,
            "versioncreated": main["versioncreated"],
            "firstcreated": main["firstcreated"],
            "headline": association.get("headline", ""),
            "description_text": association.get("description_text", ""),
            "renditions": {"baseImage": {"href": url}},
        }

        return item

    def _parse_association(self, association):
        """Parses a BBC ninjs association
        :param association:
        :return: association dict
        """
        key = association.pop("id")
        # BBC don't use 'featuremedia', they typically use 'featureimage'
        if re.match("^feature", key):
            key = "featuremedia"

        parsed = deepcopy(association)

        parsed[ITEM_TYPE] = CONTENT_TYPE.PICTURE
        url = association["renditions"]["original"]["href"]
        guid_hash = hashlib.sha1(url.encode("utf8")).hexdigest()
        parsed["guid"] = generate_guid(type=GUID_TAG, id=guid_hash)

        return key, parsed

    def _parse_subject(self, subject):
        """Parses a BBC ninjs subject
        :param subject:
        :return: subject dict
        """
        parsed = {}
        if (subject["lang"] != "en") or (subject["rel"] != "category"):
            return

        search_subject = re.sub("^category:", "", subject["code"])

        # Check if we can find the IPTC subject from the custom BBC ones
        if search_subject in self.subject_trans:
            qcode = self.subject_trans[search_subject]
            parsed["qcode"] = qcode
            parsed["name"] = subject_codes[qcode]
            return parsed

        logger.warn("Could not find subject code ({})".format(subject))

    def _parse_main(self, json):
        """Parses the main body of text and metadata
        :param json:
        :return: dict of article metadata and body
        """
        # No GUID is included so generate one from the link
        main = {}

        guid_hash = hashlib.sha1(json["uri"].encode("utf8")).hexdigest()
        main["guid"] = generate_guid(type=GUID_TAG, id=guid_hash)
        # Copy over all attributes which are the same as Superdesk's ninjs variant
        for copy_property in self.direct_copy_properties:
            if json.get(copy_property) is not None:
                main[copy_property] = json[copy_property]

        main["versioncreated"] = self._parse_date(json["versioncreated"])
        main["firstcreated"] = self._parse_date(json["firstcreated"])

        if json.get("embargotime"):
            main["embargo"] = json["embargotime"]

        main["type"] = self._convert_type(json["type"])
        return main

    def _parse_date(self, string):
        """Attempts to parse BBC ninjs time in format YYYY-MM-DDTHH:MM:SS
        :param string:
        :return: datetime
        """
        return datetime.datetime.strptime(string, "%Y-%m-%dT%H:%M:%S")

    def _convert_type(self, content_type):
        """Attempts to convert BBC's types to standard ninjs types
        :param content_type:
        :return:
        """
        if content_type == "image":
            return CONTENT_TYPE.PICTURE
        if content_type == "story" or content_type == "advisory":
            return CONTENT_TYPE.TEXT

        logger.error("could not find content type ({}), defaulting to text".format(content_type))

        return CONTENT_TYPE.TEXT


register_feed_parser(BBCNINJSFeedParser.NAME, BBCNINJSFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk import get_resource_service
from superdesk.services import Service
from superdesk.notification import push_notification
from superdesk.errors import SuperdeskApiError
from eve.utils import config
from eve.utils import ParsedRequest
from flask_babel import _
from copy import deepcopy


class ContactsService(Service):
    def get(self, req, lookup):

        # by default the response will have the inactive and not public entries filtered out
        if "all" not in req.args:
            lookup["is_active"] = True
            lookup["public"] = True
        return super().get(req, lookup)

    def on_create(self, docs):
        for doc in docs:
            self._validate_assignable(doc)

    def on_created(self, docs):
        """
        Send notification to clients that new contact(s) have been created
        :param docs:
        :return:
        """
        push_notification("contacts:create", _id=[doc.get(config.ID_FIELD) for doc in docs])

    def on_update(self, updates, original):
        item = deepcopy(original)
        item.update(updates)
        self._validate_assignable(item)

    def on_updated(self, updates, original):
        """
        Send notifification to clients that a contact has been updated
        :param updates:
        :param original:
        :return:
        """
        push_notification("contacts:update", _id=[original.get(config.ID_FIELD)])

    def on_deleted(self, doc):
        """
        Send a notification to clients that a contact has been deleted
        :param doc:
        :return:
        """
        push_notification("contacts:deleted", _id=[doc.get(config.ID_FIELD)])

    def _validate_assignable(self, contact):
        """Validates a required email address if the contact_type has assignable flag turned on"""

        if not contact or not contact.get("contact_type"):
            return

        types = get_resource_service("vocabularies").find_one(req=None, _id="contact_type")

        if not types:
            return

        contact_type = next(
            (item for item in (types.get("items") or []) if item.get("qcode") == contact.get("contact_type")), None
        )

        if not contact_type or not contact_type.get("assignable"):
            return

        if not contact.get("contact_email"):
            raise SuperdeskApiError.badRequestError(
                message=_('Contacts of type "{contact_type}" must have an email address').format(
                    contact_type=contact_type.get("name")
                )
            )


class OrganisationService(Service):
    def get(self, req, lookup):
        """
        Search for organisation matching the passed q parameter
        :param req:
        :param lookup:
        :return: List of matching organisation strings
        """
        new_req = ParsedRequest()

        q_str = "organisation:" + "* organisation:".join(req.args.get("q", "").split()) + "*"
        new_req.args = {"q": q_str, "default_operator": "AND", "projections": '{"organisation": 1}'}
        ret = super().get(new_req, lookup)

        # Remove any duplicate entries from the response
        orgs = []
        de_duped = []
        for d in ret.docs:
            if d.get("organisation") not in orgs:
                orgs.append(d.get("organisation"))
                de_duped.append(d)
        ret.docs = de_duped
        ret.hits["hits"]["total"] = len(ret.docs)
        return ret



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource


DICTIONARY_FILE = "file"


class KeywordsResource(Resource):
    """Keywords schema"""

    schema = {
        "text": {"type": "string", "required": True},
        "keywords": {
            "type": "list",
            "required": False,
            "schema": {
                "type": "dict",
                "schema": {
                    "text": {"type": "string", "required": True, "empty": False},
                    "relevance": {"type": "string", "required": True},
                },
            },
        },
    }
    item_methods = []
    resource_methods = ["POST"]
    privileges = {"POST": "archive"}



import unittest
import unittest.mock
import bson
from .base_proxy import BaseProxy


class MockDatalayer:
    find_one = None
    update = None


class BaseProxyTestCase(unittest.TestCase):
    def test_update_will_use_db_id(self):
        original = {"_id": bson.ObjectId()}
        updates = {"name": "foo", "_id": original["_id"]}
        datalayer = MockDatalayer()
        datalayer.find_one = unittest.mock.MagicMock(return_value=original)
        datalayer.update = unittest.mock.MagicMock(return_value=original)
        proxy = BaseProxy(datalayer)
        proxy.update("test", {"_id": str(original["_id"])}, updates)
        datalayer.update.assert_called_with("test", original["_id"], {"name": "foo"}, original)



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : mugur
# Creation: 2018-04-25 01:07

from eve.utils import config
from superdesk.commands.data_updates import BaseDataUpdate


class DataUpdate(BaseDataUpdate):

    resource = "vocabularies"
    update_fields = ["name", "qcode"]

    def forwards(self, mongodb_collection, mongodb_database):
        for vocabulary in mongodb_collection.find({}):
            if "schema" in vocabulary:
                schema = vocabulary["schema"]
                for field in self.update_fields:
                    if field in vocabulary["schema"] and type(vocabulary["schema"]) == dict:
                        schema[field]["required"] = True
                mongodb_collection.update({"_id": vocabulary.get(config.ID_FIELD)}, {"$set": {"schema": schema}})

    def backwards(self, mongodb_collection, mongodb_database):
        pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import inspect
import importlib
import sys
import imp
import os

from flask import current_app


def load_macros(app=None):
    if not app:
        app = current_app
    module = app.config.get("MACROS_MODULE", "superdesk.macros")
    load_module(module)


def load_module(module):
    """Loads the given module

    If the module is loaded before it will reload it

    :param module: name of he module
    """
    try:
        imp.reload(sys.modules[module])
    except (AttributeError, KeyError):
        try:
            importlib.import_module(module)
        except ImportError:
            return

    m = sys.modules[module]
    if getattr(m, "init_app", None):
        m.init_app(current_app)

    register_macros()


def register_macros():
    macro_modules = [
        sys.modules[m]
        for m in sys.modules.keys()
        if "macros." in m and "callback" in dir(sys.modules[m]) and not m.endswith("_test") and not m.startswith("__")
    ]
    # DO NOT REMOVE: This is a hack introduced long time back to solve the problem
    # where macros were not getting loaded for celery jobs.
    print(macro_modules, file=open(os.devnull, "w"))

    for macro_module in macro_modules:
        replace_type = macro_module.replace_type if hasattr(macro_module, "replace_type") else "no-replace"
        """
         replace_type:
             'no-replace': no replace action will be performed
             'simple-replace': will detect changes from backend and will perform a replace that will
                 not preserve any style
             'keep-style-replace': will detect changes from backend and will perform a replace that
                 will not preserve any set style
        """
        kwargs = {
            "name": macro_module.name,
            "callback": macro_module.callback,
            "access_type": macro_module.access_type,
            "action_type": macro_module.action_type,
            "replace_type": replace_type,
        }

        options = ["label", "order", "shortcut", "from_languages", "to_languages", "group"]
        for field in options:
            if hasattr(macro_module, field):
                kwargs[field] = getattr(macro_module, field)

        register(**kwargs)


class MacroRegister:
    """Dynamic macros registry.

    Will look for new macros whenever macros are used.
    """

    def __init__(self):
        self.macros = []

    def __iter__(self):
        """Implement for macro in macros."""
        self.index = -1
        load_macros()
        return self

    def __next__(self):
        """Implement for macro in macros."""
        self.index += 1
        try:
            return self.macros[self.index]
        except IndexError:
            raise StopIteration

    def __contains__(self, name):
        """Implement 'name' in macros.

        :param name: macro name
        """
        load_macros()
        return self.find(name) is not None

    def find(self, name):
        """Find a macro by given macro name.

        :param name: macro name
        """
        load_macros()
        for macro in self.macros:
            if macro.get("name") == name:
                return macro

    def register(self, **kwargs):
        """Register a new macro.

        :param name: unique macro name, used to identify macro
        :param label: macro label, used by client when listing macros
        :param callback: macro callback implementing functionality, should use **kwargs to be able to handle new params
        :param shortcut: default client shortcut (witch ctrl)
        :param description: macro description, using callback doctext as default
        """
        kwargs.setdefault("description", inspect.getdoc(kwargs.get("callback")))
        self.macros = [macro for macro in self.macros if macro["name"] != kwargs.get("name")]
        self.macros.append(kwargs)


macros = MacroRegister()


def register(**kwargs):
    """Alias for macro.register."""
    macros.register(**kwargs)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .area_square_feet_to_metric import square_feet_to_metric


class AreaTestCase(unittest.TestCase):
    def test_f_to_c(self):
        text = (
            "Total area is 100.50 square feet for this land. "
            "Total area is 15.7 square ft for this land. "
            "Total area is 1 Square Foot for this land. "
            "Total area is 1-16 sq-ft for this land. "
            "Total area is 16.7-Square-ft for this land. "
            "Total area is 16,500-sq. ft for this land. "
        )
        item = {"body_html": text}
        res, diff = square_feet_to_metric(item)
        self.assertEqual(diff["100.50 square feet"], "100.50 square feet (9.3 square meter)")
        self.assertEqual(diff["15.7 square ft"], "15.7 square ft (1.5 square meter)")
        self.assertEqual(diff["1 Square Foot"], "1 Square Foot (0.1 square meter)")
        self.assertEqual(diff["1-16 sq-ft"], "1-16 sq-ft (0.1-1.5 square meter)")
        self.assertEqual(diff["16.7-Square-ft"], "16.7-Square-ft (1.6 square meter)")
        self.assertEqual(diff["16,500-sq. ft"], "16,500-sq. ft (1,533 square meter)")
        self.assertEqual(res["body_html"], item["body_html"])



# This file is part of Superdesk.
#
# Copyright 2019-2020 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from inspect import isclass
from pathlib import Path
from importlib import import_module

logger = logging.getLogger(__name__)


def import_services(
    app,
    pkg_name: str,
    base_cls: type,
) -> None:
    """Import all services based on base_cls in given package

    This method will import python modules and look for a base_cls subclass there
    If found, the subclass will be instantiated
    :param app: app instance
    :param str pkg_name: name of the package to use
    :param base_cls: base class of the service
    """
    pkg = import_module(pkg_name)
    for file_path in Path(pkg.__file__).parent.glob("*.py"):
        module_name = file_path.stem
        if module_name in ("__init__", "base"):
            continue
        service_mod = import_module(pkg_name + "." + module_name)
        for obj_name in dir(service_mod):
            if obj_name.startswith("__") or obj_name == base_cls.__name__:
                continue
            obj = getattr(service_mod, obj_name)
            if not isclass(obj):
                continue
            if issubclass(obj, base_cls):
                obj(app)
                break
        else:
            logger.warning("Can't find service in module {module_name}".format(module_name=module_name))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from datetime import timedelta
from unittest.mock import patch, ANY
from superdesk.tests import TestCase
from superdesk import get_backend
from superdesk.utc import utcnow


class BackendTestCase(TestCase):
    def test_update_change_etag(self):
        backend = get_backend()
        updates = {"name": "foo"}
        item = {"name": "bar"}
        with self.app.app_context():
            ids = backend.create("ingest", [item])
            doc_old = backend.find_one("ingest", None, _id=ids[0])
            backend.update("ingest", ids[0], updates, doc_old)
            doc_new = backend.find_one("ingest", None, _id=ids[0])
            self.assertNotEqual(doc_old[self.app.config["ETAG"]], doc_new[self.app.config["ETAG"]])

    def test_check_default_dates_on_create(self):
        backend = get_backend()
        item = {"name": "foo"}
        with self.app.app_context():
            ids = backend.create("ingest", [item])
            doc = backend.find_one("ingest", None, _id=ids[0])
            self.assertIn(self.app.config["DATE_CREATED"], doc)
            self.assertIn(self.app.config["LAST_UPDATED"], doc)

    def test_check_default_dates_on_update(self):
        backend = get_backend()
        past = (utcnow() + timedelta(seconds=-2)).replace(microsecond=0)
        item = {"name": "foo", self.app.config["DATE_CREATED"]: past, self.app.config["LAST_UPDATED"]: past}
        updates = {"name": "bar"}
        with self.app.app_context():
            ids = backend.create("ingest", [item])
            doc_old = backend.find_one("ingest", None, _id=ids[0])
            backend.update("ingest", ids[0], updates, doc_old)
            doc_new = backend.find_one("ingest", None, _id=ids[0])
            date1 = doc_old[self.app.config["LAST_UPDATED"]]
            date2 = doc_new[self.app.config["LAST_UPDATED"]]
            self.assertGreaterEqual(date2, date1)
            date1 = doc_old[self.app.config["DATE_CREATED"]]
            date2 = doc_new[self.app.config["DATE_CREATED"]]
            self.assertEqual(date1, date2)

    @patch("superdesk.eve_backend.push_notification")
    def test_update_resource_push_notification(self, push_notification_mock):
        backend = get_backend()
        with self.app.app_context():
            backend.create("archive", [{"_id": "some-id"}])
            push_notification_mock.assert_called_once_with(
                "resource:created",
                resource="archive",
                _id="some-id",
            )

            backend.update(
                "archive",
                "some-id",
                {
                    "foo": 1,
                    "new": {"baz": 1},
                    "same": {"x": "y"},
                    "different": {
                        "same": 1,
                        "foo": 1,
                        "bar": {
                            "baz": 1,
                        },
                    },
                },
                {
                    "baz": 0,
                    "same": {"x": "y"},
                    "different": {
                        "same": 1,
                        "foo": 2,
                        "bar": {
                            "baz": 2,
                        },
                        "missing": 1,
                    },
                },
            )

            push_notification_mock.assert_called_with(
                "resource:updated",
                resource="archive",
                _id="some-id",
                fields={
                    "foo": 1,
                    "new": 1,
                    "new.baz": 1,
                    "different": 1,
                    "different.foo": 1,
                    "different.bar": 1,
                    "different.missing": 1,
                },
            )

            backend.delete("archive", {"_id": "some-id"})
            push_notification_mock.assert_called_with(
                "resource:deleted",
                resource="archive",
                _id="some-id",
            )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from content_api import MONGO_PREFIX


class UsersResource(Resource):
    """
    Users schema
    """

    schema = {
        "password": {"type": "string", "minlength": 8},
        "first_name": {"type": "string"},
        "last_name": {"type": "string"},
        "email": {"unique": True, "type": "string", "required": True},
        "phone": {"type": "string", "nullable": True},
        "signup_details": {"type": "dict"},
        "country": {"type": "string"},
        "company": Resource.rel("companies", embeddable=True, required=False),
        "user_type": {"type": "string", "allowed": ["administrator", "internal", "public"], "default": "public"},
        "is_validated": {"type": "boolean", "default": False},
        "is_enabled": {"type": "boolean", "default": True},
        "is_approved": {"type": "boolean", "default": False},
        "token": {
            "type": "string",
        },
        "token_expiry_date": {
            "type": "datetime",
        },
    }

    item_methods = ["GET", "PATCH", "PUT"]
    resource_methods = ["GET", "POST"]
    mongo_prefix = MONGO_PREFIX
    datasource = {"source": "users", "projection": {"password": 0}, "default_sort": [("name", 1)]}
    mongo_indexes = {"email": ([("email", 1)], {"unique": True})}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import pytz
import datetime
import superdesk
from .newsml_2_0 import NewsMLTwoFeedParser
from superdesk.io.registry import register_feed_parser
from superdesk.errors import ParserError
from superdesk.metadata.item import ITEM_TYPE
from superdesk.io.iptc import subject_codes
from superdesk.text_utils import get_word_count
from flask import current_app as app
from dateutil.parser import parse as date_parser
from superdesk.etree import parse_html, to_string


class ScoopNewsMLTwoFeedParser(NewsMLTwoFeedParser):

    NAME = "scoop_newsml2"

    label = "Scoop Media News ML-G2 Parser"

    def can_parse(self, xml):
        if xml.tag.endswith("newsMessage"):
            try:
                self.root = xml
                header = xml.find(self.qname("Header"))
                origin = header.find(self.qname("origin")).text
                if origin == "BusinessDesk":
                    return True
            except Exception:
                return False
        return False

    def parse(self, xml, provider=None):
        self.root = xml
        items = []
        try:
            for item_set in xml.findall(self.qname("itemSet")):
                for item_tree in item_set:
                    # Ignore the packageItem, it has no guid
                    if "guid" in item_tree.attrib:
                        item = self.parse_item(item_tree)
                        item["priority"] = 6
                        item["anpa_category"] = [{"qcode": "f"}]
                        item["subject"] = [{"qcode": "04000000", "name": subject_codes["04000000"]}]
                        item.setdefault("word_count", get_word_count(item["body_html"]))
                        # Hard code the urgency
                        item["urgency"] = 3
                        # Dateline is always Wellington in NZ
                        located = [
                            c
                            for c in app.locators.find_cities(country_code="NZ", state_code="NZ.G2")
                            if c.get("city", "").lower() == "wellington"
                        ]
                        if len(located) == 1:
                            item["dateline"] = dict()
                            item["dateline"]["located"] = located[0]

                        if item.get("body_html") and item["dateline"]:
                            parsed = parse_html(item.get("body_html"), content="xml")
                            pars = parsed.xpath("//p")
                            for par in pars:
                                if not par.text:
                                    continue
                                # check the first par for a byline
                                if pars.index(par) == 0 and par.text.startswith("By "):
                                    item["byline"] = par.text.replace("By ", "")
                                    par.getparent().remove(par)
                                date, source, the_rest = par.text.partition(" (BusinessDesk) - ")
                                if source:
                                    item["dateline"]["date"] = date_parser(date, fuzzy=True)
                                    par.text = the_rest
                                # remove the signoff if in the last par
                                if par.text == "(BusinessDesk)" and pars.index(par) + 1 == len(pars):
                                    par.getparent().remove(par)
                            item["body_html"] = to_string(parsed, remove_root_div=True)
                        locator_map = superdesk.get_resource_service("vocabularies").find_one(req=None, _id="locators")
                        if locator_map:
                            item["place"] = [x for x in locator_map.get("items", []) if x["qcode"].upper() == "NZ"]

                        items.append(item)
            return items
        except Exception as ex:
            raise ParserError.newsmlTwoParserError(ex, provider)

    def parse_header(self, tree):
        """Parse header element, it seems that the header tag is in camel case

        :param tree:
        :return: dict
        """
        header = tree.find(self.qname("Header"))
        priority = 5
        if header is not None:
            priority = self.map_priority(header.find(self.qname("priority")).text)

        return {"priority": priority}

    def parse_item_meta(self, tree, item):
        """Parse itemMeta tag"""
        meta = tree.find(self.qname("itemMeta"))
        item[ITEM_TYPE] = meta.find(self.qname("itemClass")).attrib["qcode"].split(":")[1]
        item["versioncreated"] = self.datetime(meta.find(self.qname("versionCreated")).text)
        item["firstcreated"] = item["versioncreated"]
        item["pubstatus"] = (meta.find(self.qname("pubStatus")).attrib["qcode"].split(":")[1]).lower()
        item["ednote"] = meta.find(self.qname("edNote")).text if meta.find(self.qname("edNote")) is not None else ""

    def datetime(self, string):
        """Convert the date string parsed from the source file to a datetime, assumes that the time is local to NZ

        :param string:
        :return:
        """
        local_dt = datetime.datetime.strptime(string, "%Y-%m-%d %H:%M:%S")
        local_tz = pytz.timezone("Pacific/Auckland")
        nz_dt = local_tz.localize(local_dt, is_dst=None)
        return nz_dt.astimezone(pytz.utc)

    def parse_content_set(self, tree, item):
        """Parse out the nitf like content.

        :param tree:
        :param item:
        :return: item populated with a headline and body_html
        """
        for content in tree.find(self.qname("contentSet")):
            if content.tag == self.qname("inlineXML") and content.attrib["contenttype"] == "application/nitf+xml":
                nitf = content.find(self.qname("nitf"))
                head = nitf.find(self.qname("head"))
                item["headline"] = head.find(self.qname("title")).text
                body = nitf.find(self.qname("body"))
                content = self.parse_inline_content(body)
                item["body_html"] = content.get("content")

    def parse_inline_content(self, tree):
        body = tree.find(self.qname("body.content"))
        elements = []
        for elem in body:
            if elem.text:
                tag = elem.tag.rsplit("}")[1]
                elements.append("<%s>%s</%s>" % (tag, elem.text, tag))

        content = dict()
        content["content"] = "\n".join(elements)
        return content


register_feed_parser(ScoopNewsMLTwoFeedParser.NAME, ScoopNewsMLTwoFeedParser())



import unittest
from datetime import datetime, timedelta

from apps.rules.routing_rules import Weekdays


class WeekdaysTestCase(unittest.TestCase):
    def test_is_valid_schedule(self):
        self.assertTrue(Weekdays.is_valid_schedule(["MON", "TUE"]))
        self.assertFalse(Weekdays.is_valid_schedule(["MON", "SRC"]))

    def test_is_scheduled(self):
        now = datetime.now()
        day = now.strftime("%A")[:3]

        self.assertFalse(Weekdays.is_scheduled_day(now, []))
        self.assertTrue(Weekdays.is_scheduled_day(now, [day]))
        self.assertFalse(Weekdays.is_scheduled_day(now + timedelta(days=1), [day]))

    def test_weekday_dayname(self):
        now = datetime.now()
        day = now.strftime("%A")[:3].upper()
        self.assertEqual(day, Weekdays.dayname(now))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
import superdesk.signals as signals

from eve.utils import config
from eve.versioning import resolve_document_version
from flask import request, current_app as app
from copy import deepcopy

from apps.tasks import send_to, apply_onstage_rule
from apps.desks import DeskTypes
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.metadata.item import (
    ITEM_STATE,
    CONTENT_STATE,
    SIGN_OFF,
    LAST_AUTHORING_DESK,
    LAST_PRODUCTION_DESK,
    LAST_DESK,
    DESK_HISTORY,
)
from superdesk.metadata.packages import REFS, GROUPS, RESIDREF
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.metadata.utils import item_url
from apps.archive.common import (
    insert_into_versions,
    item_operations,
    ITEM_OPERATION,
    set_sign_off,
    get_user,
    convert_task_attributes_to_objectId,
)
from apps.archive.archive import SOURCE as ARCHIVE
from superdesk.workflow import is_workflow_state_transition_valid
from apps.content import push_item_move_notification
from superdesk.lock import lock, unlock
from flask_babel import _

ITEM_MOVE = "move"
item_operations.append(ITEM_MOVE)


class MoveResource(Resource):
    endpoint_name = "move"
    resource_title = endpoint_name

    schema = {
        "task": {
            "type": "dict",
            "required": False,
            "schema": {
                "desk": Resource.rel("desks", False, required=True),
                "stage": Resource.rel("stages", False, required=True),
            },
        },
        "allPackageItems": {"type": "boolean", "required": False, "nullable": True},
    }

    url = "archive/<{0}:guid>/move".format(item_url)

    resource_methods = ["POST"]
    item_methods = []

    privileges = {"POST": "archive"}


class MoveService(BaseService):
    def create(self, docs, **kwargs):
        guid_of_item_to_be_moved = request.view_args["guid"]
        guid_of_moved_items = []

        # set the lock_id it per item
        lock_id = "item_move {}".format(guid_of_item_to_be_moved)

        if not lock(lock_id, expire=5):
            raise SuperdeskApiError.forbiddenError(message=_("Item is locked for move by another user."))

        try:
            # doc represents the target desk and stage
            doc = docs[0]
            moved_item = self.move_content(guid_of_item_to_be_moved, doc)
            guid_of_moved_items.append(moved_item.get(config.ID_FIELD))

            if moved_item.get("type", None) == "composite" and doc.get("allPackageItems", False):
                try:
                    item_lock_id = None
                    for item_id in (
                        ref[RESIDREF]
                        for group in moved_item.get(GROUPS, [])
                        for ref in group.get(REFS, [])
                        if RESIDREF in ref
                    ):
                        item_lock_id = "item_move {}".format(item_id)
                        if lock(item_lock_id, expire=5):
                            item = self.move_content(item_id, doc)
                            guid_of_moved_items.append(item.get(config.ID_FIELD))
                            unlock(item_lock_id, remove=True)
                            item_lock_id = None
                        else:
                            item_lock_id = None
                finally:
                    if item_lock_id:
                        unlock(item_lock_id, remove=True)

            return guid_of_moved_items
        finally:
            unlock(lock_id, remove=True)

    def move_content(self, id, doc):
        archive_service = get_resource_service(ARCHIVE)
        archived_doc = archive_service.find_one(req=None, _id=id)

        if not archived_doc:
            raise SuperdeskApiError.notFoundError(_("Fail to found item with guid: {guid}").format(guid=id))

        self._validate(archived_doc, doc)
        self._move(archived_doc, doc)

        # get the recent updates again
        archived_doc = archive_service.find_one(req=None, _id=id)
        # finally apply any on stage rules/macros
        apply_onstage_rule(archived_doc, id)

        return archived_doc

    def _move(self, archived_doc, doc):
        archive_service = get_resource_service(ARCHIVE)
        original = deepcopy(archived_doc)
        user = get_user()
        send_to(
            doc=archived_doc,
            desk_id=doc.get("task", {}).get("desk"),
            stage_id=doc.get("task", {}).get("stage"),
            user_id=user.get(config.ID_FIELD),
        )
        if archived_doc[ITEM_STATE] not in (
            {
                CONTENT_STATE.PUBLISHED,
                CONTENT_STATE.SCHEDULED,
                CONTENT_STATE.KILLED,
                CONTENT_STATE.RECALLED,
                CONTENT_STATE.CORRECTION,
            }
        ):
            archived_doc[ITEM_STATE] = CONTENT_STATE.SUBMITTED
        archived_doc[ITEM_OPERATION] = ITEM_MOVE
        # set the change in desk type when content is moved.
        self.set_change_in_desk_type(archived_doc, original)
        archived_doc.pop(SIGN_OFF, None)
        set_sign_off(archived_doc, original=original)
        convert_task_attributes_to_objectId(archived_doc)
        resolve_document_version(archived_doc, ARCHIVE, "PATCH", original)

        del archived_doc[config.ID_FIELD]
        del archived_doc[config.ETAG]  # force etag update

        signals.item_move.send(self, item=archived_doc, original=original)
        archive_service.update(original[config.ID_FIELD], archived_doc, original)

        insert_into_versions(id_=original[config.ID_FIELD])
        push_item_move_notification(original, archived_doc)
        app.on_archive_item_updated(archived_doc, original, ITEM_MOVE)

        # make sure `item._id` is there in signal
        moved_item = archived_doc.copy()
        moved_item[config.ID_FIELD] = original[config.ID_FIELD]
        signals.item_moved.send(self, item=moved_item, original=original)

    def _validate(self, archived_doc, doc):
        """Validate that the item can be move.

        :param dict archived_doc: item to be moved
        :param dict doc: new location details
        """
        current_stage_of_item = archived_doc.get("task", {}).get("stage")
        if current_stage_of_item and str(current_stage_of_item) == str(doc.get("task", {}).get("stage")):
            raise SuperdeskApiError.preconditionFailedError(message=_("Move is not allowed within the same stage."))
        if not is_workflow_state_transition_valid("submit_to_desk", archived_doc[ITEM_STATE]):
            raise InvalidStateTransitionError()

    def set_change_in_desk_type(self, updated, original):
        """Detects if the change in the desk is between authoring to production (and vice versa).

        Sets the field 'last_production_desk' and 'last_authoring_desk'.

        :param dict updated: document to be saved
        :param dict original: original document
        """
        old_desk_id = str(original.get("task", {}).get("desk", ""))
        new_desk_id = str(updated.get("task", {}).get("desk", ""))
        if old_desk_id and old_desk_id != new_desk_id:
            old_desk = get_resource_service("desks").find_one(req=None, _id=old_desk_id)
            new_desk = get_resource_service("desks").find_one(req=None, _id=new_desk_id)
            if old_desk and new_desk and old_desk.get("desk_type", "") != new_desk.get("desk_type", ""):
                if new_desk.get("desk_type") == DeskTypes.production.value:
                    updated["task"][LAST_AUTHORING_DESK] = old_desk_id
                else:
                    updated["task"][LAST_PRODUCTION_DESK] = old_desk_id
            updated["task"][LAST_DESK] = old_desk_id
            updated["task"].setdefault(DESK_HISTORY, [])
            if old_desk_id not in updated["task"][DESK_HISTORY]:
                updated["task"][DESK_HISTORY].append(old_desk_id)


superdesk.workflow_action(
    name="submit_to_desk",
    include_states=[
        "draft",
        "fetched",
        "routed",
        "submitted",
        "in_progress",
        "published",
        "scheduled",
        "unpublished",
        "correction",
    ],
    privileges=["archive"],
)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime

from superdesk.etree import etree
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.io.iptc import subject_codes
from superdesk.errors import ParserError
from superdesk.metadata.item import ITEM_TYPE
from superdesk.metadata.item import CONTENT_TYPE
from superdesk.utc import utc


class NewsMLOneFeedParser(XMLFeedParser):
    """
    Feed Parser which can parse if the feed is in NewsML 1.2 format.
    """

    NAME = "newsml12"

    label = "News ML 1.2 Parser"

    def can_parse(self, xml):
        return xml.tag == "NewsML" and xml.get("Version", "") == "1.2"

    def parse(self, xml, provider=None):
        item = {}
        try:
            self.root = xml
            parsed_el = xml.find("NewsItem/NewsComponent/AdministrativeMetadata/Source")
            if parsed_el is not None:
                item["original_source"] = parsed_el.find("Party").get("FormalName", "")

            parsed_el = xml.find("NewsEnvelope/TransmissionId")
            if parsed_el is not None:
                item["ingest_provider_sequence"] = parsed_el.text

            parsed_el = xml.find("NewsEnvelope/Priority")
            item["priority"] = self.map_priority(parsed_el.text if parsed_el is not None else None)

            self.parse_news_identifier(item, xml)
            self.parse_newslines(item, xml)
            self.parse_news_management(item, xml)

            parsed_el = xml.findall("NewsItem/NewsComponent/DescriptiveMetadata/Language")
            if parsed_el is not None:
                language = self.parse_attributes_as_dictionary(parsed_el)
                item["language"] = language[0]["FormalName"] if len(language) else ""

            keywords = xml.findall("NewsItem/NewsComponent/DescriptiveMetadata/Property")
            item["keywords"] = self.parse_attribute_values(keywords, "Keyword")

            subjects = xml.findall("NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/SubjectDetail")
            subjects += xml.findall("NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/SubjectMatter")
            subjects += xml.findall("NewsItem/NewsComponent/DescriptiveMetadata/SubjectCode/Subject")

            item["subject"] = self.format_subjects(subjects)

            # item['ContentItem'] = self.parse_attributes_as_dictionary(
            #    tree.find('NewsItem/NewsComponent/ContentItem'))
            # item['Content'] = etree.tostring(
            # tree.find('NewsItem/NewsComponent/ContentItem/DataContent/nitf/body/body.content'))

            self.parse_content(item, xml)

            parsed_el = xml.findall("NewsItem/NewsComponent/ContentItem/Characteristics/Property")
            characteristics = self.parse_attribute_values(parsed_el, "Words")
            item["word_count"] = characteristics[0] if len(characteristics) else None

            parsed_el = xml.find("NewsItem/NewsComponent/RightsMetadata/UsageRights/UsageType")
            if parsed_el is not None:
                item.setdefault("usageterms", parsed_el.text)

            parsed_el = xml.findall("NewsItem/NewsComponent/DescriptiveMetadata/Genre")
            if parsed_el is not None:
                item["genre"] = []
                for el in parsed_el:
                    item["genre"].append({"name": el.get("FormalName")})

            return self.populate_fields(item)
        except Exception as ex:
            raise ParserError.newsmlOneParserError(ex, provider)

    def parse_content(self, item, xml):
        item["body_html"] = (
            etree.tostring(
                xml.find("NewsItem/NewsComponent/ContentItem/DataContent/nitf/body/body.content"),
                encoding="unicode",
            )
            .replace("<body.content>", "")
            .replace("</body.content>", "")
        )

    def parse_elements(self, tree):
        items = {}
        for item in tree:
            if item.text is None:
                # read the attribute for the item
                if item.tag != "HeadLine":
                    items[item.tag] = item.attrib
            else:
                # read the value for the item
                items[item.tag] = item.text
        return items

    def parse_multivalued_elements(self, tree):
        items = {}
        for item in tree:
            if item.tag not in items:
                items[item.tag] = [item.text]
            else:
                items[item.tag].append(item.text)

        return items

    def parse_attributes_as_dictionary(self, items):
        attributes = [item.attrib for item in items]
        return attributes

    def parse_attribute_values(self, items, attribute):
        attributes = []
        for item in items:
            if item.attrib["FormalName"] == attribute:
                attributes.append(item.attrib["Value"])
        return attributes

    def datetime(self, string):
        try:
            return datetime.datetime.strptime(string, "%Y%m%dT%H%M%S+0000")
        except ValueError:
            return datetime.datetime.strptime(string, "%Y%m%dT%H%M%SZ").replace(tzinfo=utc)

    def populate_fields(self, item):
        item[ITEM_TYPE] = CONTENT_TYPE.TEXT
        return item

    def parse_news_identifier(self, item, tree):
        parsed_el = self.parse_elements(tree.find("NewsItem/Identification/NewsIdentifier"))
        item["guid"] = parsed_el["PublicIdentifier"]
        item["version"] = parsed_el["RevisionId"]

    def parse_news_management(self, item, tree):
        parsed_el = self.parse_elements(tree.find("NewsItem/NewsManagement"))
        try:
            item["urgency"] = int(parsed_el["Urgency"]["FormalName"])
        except KeyError:
            pass
        item["versioncreated"] = self.datetime(parsed_el["ThisRevisionCreated"])
        item["firstcreated"] = self.datetime(parsed_el["FirstCreated"])
        item["pubstatus"] = (parsed_el["Status"]["FormalName"]).lower()
        # if parsed_el['NewsItemType']['FormalName'] == 'Alert':
        #    parsed_el['headline'] = 'Alert'

    def parse_newslines(self, item, tree):
        parsed_el = self.parse_elements(tree.find("NewsItem/NewsComponent/NewsLines"))

        self.set_dateline(item, text=parsed_el.get("DateLine", ""))

        item["headline"] = parsed_el.get("HeadLine", "")
        item["slugline"] = parsed_el.get("SlugLine", "")
        item["byline"] = parsed_el.get("ByLine", "")

        return True

    def format_subjects(self, subjects):
        """Map the ingested Subject Codes to their corresponding names as per IPTC Specification.

        :param subjects: list of dicts where each dict gives the category the article is mapped to.
        :type subjects: list
        :returns [{"qcode": "01001000", "name": "archaeology"}, {"qcode": "01002000", "name": "architecture"}]
        :rtype list
        """

        formatted_subjects = []

        def is_not_formatted(qcode):
            for formatted_subject in formatted_subjects:
                if formatted_subject["qcode"] == qcode:
                    return False

            return True

        for subject in subjects:
            formal_name = subject.get("FormalName")
            if formal_name and is_not_formatted(formal_name):
                formatted_subjects.append({"qcode": formal_name, "name": subject_codes.get(formal_name, "")})

        return formatted_subjects


register_feed_parser(NewsMLOneFeedParser.NAME, NewsMLOneFeedParser())



from flask_babel import _

import superdesk
from apps.archive.common import ARCHIVE
from superdesk import config
from superdesk.errors import SuperdeskApiError
from superdesk.media.video_editor import VideoEditorWrapper
from superdesk.metadata.utils import item_url


TIMELINE_THUMBNAILS_AMOUNT = 60


class VideoEditService(superdesk.Service):
    """
    Use video server for editing video.
    """

    video_editor = VideoEditorWrapper()

    def create(self, docs, **kwargs):
        ids = []
        for doc in docs:
            item = doc.get("item")
            item_id = item[config.ID_FIELD]
            renditions = item["renditions"]
            video_id = renditions["original"].get("video_editor_id")
            if not video_id:
                raise SuperdeskApiError.badRequestError(message=_('"video_editor_id" is required'))
            if "capture" not in doc and "edit" not in doc:
                raise SuperdeskApiError.badRequestError(message=_('"capture" or "edit" is required'))

            # push task capture preview thumbnail to video server
            if "capture" in doc:
                capture = doc.pop("capture")
                project = self.video_editor.capture_preview_thumbnail(
                    video_id, position=capture.get("position"), crop=capture.get("crop"), rotate=capture.get("rotate")
                )
                renditions.setdefault("viewImage", {}).update(
                    {
                        "href": project["thumbnails"]["preview"].get("url"),
                        "mimetype": project["thumbnails"]["preview"].get("mime_type", "image/png"),
                    }
                )

                # clean up old thumbnails
                renditions.pop("baseImage", None)
                renditions.pop("thumbnail", None)
            # push task edit video to video server
            if "edit" in doc:
                edit = doc.pop("edit")
                project = self.video_editor.edit(video_id, edit)
                renditions.setdefault("original", {}).update(
                    {
                        "href": project["url"],
                        "mimetype": project.get("mime_type", "video/mp4"),
                        "version": project["version"] + 1,
                        "video_editor_id": project.get("_id"),
                    }
                )

            original_item = super().find_one(req=None, _id=item_id)
            updates = self.system_update(id=item_id, updates={"renditions": renditions}, original=original_item)
            item.update(updates)
            ids.append(item_id)
        return ids

    def find_one(self, req, **lookup):
        res = super().find_one(req, **lookup)
        if req is None:
            return res

        video_id = res["renditions"]["original"]["video_editor_id"]
        if req.args.get("action") == "timeline":
            response = self.video_editor.create_timeline_thumbnails(
                video_id, req.args.get("amount", TIMELINE_THUMBNAILS_AMOUNT)
            )
            return {config.ID_FIELD: video_id, **response}
        res["project"] = self.video_editor.find_one(video_id)
        return res

    def on_replace(self, document, original):
        """
        Override to upload thumbnails
        """
        if not document.get("file"):
            return
        # avoid dump file storage
        file = document.pop("file")
        project = original.pop("project")
        data = self.video_editor.upload_preview_thumbnail(project.get("_id"), file)
        document.update(original)
        renditions = document.get("renditions", {})
        renditions.setdefault("viewImage", {}).update(
            {
                "href": data.get("url"),
                "mimetype": data.get("mimetype"),
            }
        )
        renditions.pop("baseImage", None)
        renditions.pop("thumbnail", None)
        document.update({"renditions": renditions})
        return document


class VideoEditResource(superdesk.Resource):
    item_methods = ["GET", "PUT"]
    resource_methods = ["POST"]
    privileges = {
        "POST": ARCHIVE,
        "PUT": ARCHIVE,
    }
    datasource = {
        "projection": {
            "processing": 1,
            "thumbnails": 1,
            "project": 1,
            "renditions": 1,
        }
    }
    item_url = item_url
    schema = {
        "file": {"type": "file"},
        "item": {
            "type": "dict",
            "required": False,
            "empty": True,
            "schema": {
                config.ID_FIELD: {
                    "type": "string",
                    "required": True,
                },
                "renditions": {
                    "type": "dict",
                    "required": True,
                    "allow_unknown": True,
                    "schema": {"original": {"type": "dict", "required": True, "empty": False}},
                },
            },
        },
        "edit": {
            "type": "dict",
            "required": False,
            "empty": False,
            "schema": {
                "trim": {
                    "required": False,
                    "regex": "^\\d+\\.?\\d*,\\d+\\.?\\d*$",
                },
                "rotate": {"type": "integer", "required": False, "allowed": [-270, -180, -90, 90, 180, 270]},
                "scale": {"type": "integer", "required": False},
                "crop": {"required": False, "regex": "^\\d+,\\d+,\\d+,\\d+$"},
            },
        },
        "capture": {
            "type": "dict",
            "required": False,
            "empty": False,
            "schema": {
                "position": {
                    "required": False,
                    "type": "float",
                },
                "rotate": {"type": "integer", "required": False, "allowed": [-270, -180, -90, 90, 180, 270]},
                "scale": {"type": "integer", "required": False},
                "crop": {"required": False, "regex": "^\\d+,\\d+,\\d+,\\d+$"},
            },
        },
    }


def init_app(app):
    video_edit_service = VideoEditService(ARCHIVE, backend=superdesk.get_backend())
    VideoEditResource("video_edit", app=app, service=video_edit_service)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging
import superdesk

from flask import json
from flask_babel import _
from eve.utils import config

from superdesk import get_resource_service
from superdesk.utc import utcnow
from superdesk.errors import SuperdeskApiError, ProviderError
from superdesk.resource import Resource
from apps.archive.common import ARCHIVE, insert_into_versions, fetch_item

logger = logging.getLogger(__name__)


class ProviderNotFoundError(SuperdeskApiError):
    pass


class SearchIngestResource(superdesk.Resource):
    resource_methods = ["GET", "POST"]
    schema = {
        "guid": {"type": "string", "required": True},
        "desk": Resource.rel("desks", False, nullable=True),
        "stage": Resource.rel("stages", False, nullable=True),
    }


class SearchIngestService(superdesk.Service):
    def __init__(self, datasource=None, backend=None, source=None):
        super().__init__(datasource, backend)
        self.source = source

    def get_provider(self):
        provider = get_resource_service("search_providers").find_one(source=self.source, req=None)
        if provider and "config" in provider and "username" in provider["config"]:
            self.backend.set_credentials(provider["config"]["username"], provider["config"].get("password", ""))
        return provider

    def fetch(self, guid):
        return self.backend.find_one_raw(guid, guid)

    def create(self, docs, **kwargs):
        new_guids = []
        provider = self.get_provider()
        for doc in docs:
            if not doc.get("desk"):
                # if no desk is selected then it is bad request
                raise SuperdeskApiError.badRequestError(_("Destination desk cannot be empty."))
            try:
                archived_doc = self.fetch(doc["guid"])
            except FileNotFoundError as ex:
                raise ProviderError.externalProviderError(ex, provider)

            dest_doc = fetch_item(archived_doc, doc.get("desk"), doc.get("stage"), state=doc.get("state"))
            new_guids.append(dest_doc["guid"])

            if provider:
                dest_doc["ingest_provider"] = str(provider[superdesk.config.ID_FIELD])

            superdesk.get_resource_service(ARCHIVE).post([dest_doc])
            insert_into_versions(dest_doc.get("_id"))

        if new_guids:
            get_resource_service("search_providers").system_update(
                provider.get(config.ID_FIELD), {"last_item_update": utcnow()}, provider
            )

        return new_guids

    def get(self, req, lookup):
        provider = self.get_provider()
        if provider:
            query = self._get_query(req)
            results = self.backend.find(self.source, query, None)
            for doc in results.docs:
                doc["ingest_provider"] = str(provider[superdesk.config.ID_FIELD])
            return results
        else:
            raise ProviderNotFoundError(_("provider not found source={source}").format(source=self.source))

    def fetch_rendition(self, rendition):
        """Get file stream for given rendition specs.

        Rendition should be from item that was fetched via this service get method.
        It can use api authentication if needed to fetch this binary.

        :param rendition: rendition dict
        """
        self.get_provider()
        return self.backend.fetch_file(rendition.get("href"))

    def _get_query(self, req):
        args = getattr(req, "args", {})
        query = json.loads(args.get("source")) if args.get("source") else {"query": {"filtered": {}}}
        return query
